// Copyright 2020 The Gitea Authors. All rights reserved.
// SPDX-License-Identifier: MIT

package login

import (
	"context"

	"gitea.dev/tea/cmd/flags"
	"gitea.dev/tea/modules/config"
	"gitea.dev/tea/modules/print"

	"github.com/urfave/cli/v3"
)

// CmdLoginList represents to login a gitea server.
var CmdLoginList = cli.Command{
	Name:        "list",
	Aliases:     []string{"ls"},
	Usage:       "List Gitea logins",
	Description: `List Gitea logins`,
	ArgsUsage:   " ", // command does not accept arguments
	Action:      RunLoginList,
	Flags:       []cli.Flag{&flags.OutputFlag},
}

// RunLoginList list all logins
func RunLoginList(requestCtx context.Context, cmd *cli.Command) error {
	logins, err := config.GetLogins()
	if err != nil {
		return err
	}
	return print.LoginsList(logins, cmd.String("output"))
}
