// Copyright 2021 The Gitea Authors. All rights reserved.
// SPDX-License-Identifier: MIT

package cmd

import (
	stdctx "context"

	"github.com/urfave/cli/v3"

	"gitea.dev/tea/cmd/admin/users"
	"gitea.dev/tea/modules/context"
	"gitea.dev/tea/modules/print"
)

// CmdAdmin represents the namespace of admin commands.
// The command itself has no functionality, but hosts subcommands.
var CmdAdmin = cli.Command{
	Name:     "admin",
	Usage:    "Operations requiring admin access on the Gitea instance",
	Aliases:  []string{"a"},
	Category: catMisc,
	Action: func(requestCtx stdctx.Context, cmd *cli.Command) error {
		return cli.ShowSubcommandHelp(cmd)
	},
	Commands: []*cli.Command{
		&cmdAdminUsers,
	},
}

var cmdAdminUsers = cli.Command{
	Name:    "users",
	Aliases: []string{"u"},
	Usage:   "Manage registered users",
	Action: func(ctx stdctx.Context, cmd *cli.Command) error {
		if cmd.Args().Len() == 1 {
			return runAdminUserDetail(ctx, cmd, cmd.Args().First())
		}
		return users.RunUserList(ctx, cmd)
	},
	Commands: []*cli.Command{
		&users.CmdUserList,
		&users.CmdUserCreate,
		&users.CmdUserEdit,
		&users.CmdUserDelete,
	},
	Flags: users.CmdUserList.Flags,
}

func runAdminUserDetail(requestCtx stdctx.Context, cmd *cli.Command, u string) error {
	ctx, err := context.InitCommand(cmd)
	if err != nil {
		return err
	}
	client := ctx.Login.Client()
	user, _, err := client.Users.GetUserInfo(requestCtx, u)
	if err != nil {
		return err
	}

	print.UserDetails(user)
	return nil
}
