// Code generated by go generate; DO NOT EDIT.

package gitea

import (
	"context"
	"io"
)

// Deprecated: use `client.Actions.CreateGlobalRunnerRegistrationToken` instead.
func (c *Client) CreateAdminActionRunnerRegistrationToken(ctx context.Context) (*RegistrationToken, *Response, error) {
	c.initServices()
	return c.Actions.CreateGlobalRunnerRegistrationToken(ctx)
}

// Deprecated: use `client.Actions.CreateOrgRunnerRegistrationToken` instead.
func (c *Client) CreateOrgActionRunnerRegistrationToken(ctx context.Context, org string) (*RegistrationToken, *Response, error) {
	c.initServices()
	return c.Actions.CreateOrgRunnerRegistrationToken(ctx, org)
}

// Deprecated: use `client.Actions.CreateOrgSecret` instead.
func (c *Client) CreateOrgActionSecret(ctx context.Context, org, secretName string, opt CreateOrUpdateSecretOption) (*Response, error) {
	c.initServices()
	return c.Actions.CreateOrgSecret(ctx, org, secretName, opt)
}

// Deprecated: use `client.Actions.CreateOrgVariable` instead.
func (c *Client) CreateOrgActionVariable(ctx context.Context, org, name string, opt CreateActionsVariableOption) (*Response, error) {
	c.initServices()
	return c.Actions.CreateOrgVariable(ctx, org, name, opt)
}

// Deprecated: use `client.Actions.CreateRepoRunnerRegistrationToken` instead.
func (c *Client) CreateRepoActionRunnerRegistrationToken(ctx context.Context, owner, repo string) (*RegistrationToken, *Response, error) {
	c.initServices()
	return c.Actions.CreateRepoRunnerRegistrationToken(ctx, owner, repo)
}

// Deprecated: use `client.Actions.CreateRepoSecret` instead.
func (c *Client) CreateRepoActionSecret(ctx context.Context, user, repo, secretName string, opt CreateOrUpdateSecretOption) (*Response, error) {
	c.initServices()
	return c.Actions.CreateRepoSecret(ctx, user, repo, secretName, opt)
}

// Deprecated: use `client.Actions.CreateRepoVariable` instead.
func (c *Client) CreateRepoActionVariable(ctx context.Context, user, repo, variableName, value string) (*Response, error) {
	c.initServices()
	return c.Actions.CreateRepoVariable(ctx, user, repo, variableName, value)
}

// Deprecated: use `client.Actions.CreateUserRunnerRegistrationToken` instead.
func (c *Client) CreateUserActionRunnerRegistrationToken(ctx context.Context) (*RegistrationToken, *Response, error) {
	c.initServices()
	return c.Actions.CreateUserRunnerRegistrationToken(ctx)
}

// Deprecated: use `client.Actions.CreateUserSecret` instead.
func (c *Client) CreateUserActionSecret(ctx context.Context, secretName string, opt CreateOrUpdateSecretOption) (*Response, error) {
	c.initServices()
	return c.Actions.CreateUserSecret(ctx, secretName, opt)
}

// Deprecated: use `client.Actions.CreateUserVariable` instead.
func (c *Client) CreateUserActionVariable(ctx context.Context, variableName string, opt CreateActionsVariableOption) (*Response, error) {
	c.initServices()
	return c.Actions.CreateUserVariable(ctx, variableName, opt)
}

// Deprecated: use `client.Actions.DeleteGlobalRunner` instead.
func (c *Client) DeleteAdminActionRunner(ctx context.Context, runnerID int64) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteGlobalRunner(ctx, runnerID)
}

// Deprecated: use `client.Actions.DeleteOrgRunner` instead.
func (c *Client) DeleteOrgActionRunner(ctx context.Context, org string, runnerID int64) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteOrgRunner(ctx, org, runnerID)
}

// Deprecated: use `client.Actions.DeleteOrgSecret` instead.
func (c *Client) DeleteOrgActionSecret(ctx context.Context, org, secretName string) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteOrgSecret(ctx, org, secretName)
}

// Deprecated: use `client.Actions.DeleteOrgVariable` instead.
func (c *Client) DeleteOrgActionVariable(ctx context.Context, org, name string) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteOrgVariable(ctx, org, name)
}

// Deprecated: use `client.Actions.DeleteRepoArtifact` instead.
func (c *Client) DeleteRepoActionArtifact(ctx context.Context, owner, repo string, artifactID int64) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteRepoArtifact(ctx, owner, repo, artifactID)
}

// Deprecated: use `client.Actions.DeleteRepoRun` instead.
func (c *Client) DeleteRepoActionRun(ctx context.Context, owner, repo string, runID int64) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteRepoRun(ctx, owner, repo, runID)
}

// Deprecated: use `client.Actions.DeleteRepoRunner` instead.
func (c *Client) DeleteRepoActionRunner(ctx context.Context, owner, repo string, runnerID int64) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteRepoRunner(ctx, owner, repo, runnerID)
}

// Deprecated: use `client.Actions.DeleteRepoSecret` instead.
func (c *Client) DeleteRepoActionSecret(ctx context.Context, user, repo, secretName string) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteRepoSecret(ctx, user, repo, secretName)
}

// Deprecated: use `client.Actions.DeleteRepoVariable` instead.
func (c *Client) DeleteRepoActionVariable(ctx context.Context, user, reponame, variableName string) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteRepoVariable(ctx, user, reponame, variableName)
}

// Deprecated: use `client.Actions.DeleteUserRunner` instead.
func (c *Client) DeleteUserActionRunner(ctx context.Context, runnerID int64) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteUserRunner(ctx, runnerID)
}

// Deprecated: use `client.Actions.DeleteUserSecret` instead.
func (c *Client) DeleteUserActionSecret(ctx context.Context, secretName string) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteUserSecret(ctx, secretName)
}

// Deprecated: use `client.Actions.DeleteUserVariable` instead.
func (c *Client) DeleteUserActionVariable(ctx context.Context, variableName string) (*Response, error) {
	c.initServices()
	return c.Actions.DeleteUserVariable(ctx, variableName)
}

// Deprecated: use `client.Actions.DisableRepoWorkflow` instead.
func (c *Client) DisableRepoActionWorkflow(ctx context.Context, owner, repo, workflowID string) (*Response, error) {
	c.initServices()
	return c.Actions.DisableRepoWorkflow(ctx, owner, repo, workflowID)
}

// Deprecated: use `client.Actions.DispatchRepoWorkflow` instead.
func (c *Client) DispatchRepoActionWorkflow(ctx context.Context, owner, repo, workflowID string, opt CreateActionsWorkflowDispatchOption, returnRunDetails bool) (*RunDetails, *Response, error) {
	c.initServices()
	return c.Actions.DispatchRepoWorkflow(ctx, owner, repo, workflowID, opt, returnRunDetails)
}

// Deprecated: use `client.Actions.EnableRepoWorkflow` instead.
func (c *Client) EnableRepoActionWorkflow(ctx context.Context, owner, repo, workflowID string) (*Response, error) {
	c.initServices()
	return c.Actions.EnableRepoWorkflow(ctx, owner, repo, workflowID)
}

// Deprecated: use `client.Actions.GetGlobalRunner` instead.
func (c *Client) GetAdminActionRunner(ctx context.Context, runnerID int64) (*ActionsRunner, *Response, error) {
	c.initServices()
	return c.Actions.GetGlobalRunner(ctx, runnerID)
}

// Deprecated: use `client.Actions.GetOrgRunner` instead.
func (c *Client) GetOrgActionRunner(ctx context.Context, org string, runnerID int64) (*ActionsRunner, *Response, error) {
	c.initServices()
	return c.Actions.GetOrgRunner(ctx, org, runnerID)
}

// Deprecated: use `client.Actions.GetOrgVariable` instead.
func (c *Client) GetOrgActionVariable(ctx context.Context, org, name string) (*ActionsVariable, *Response, error) {
	c.initServices()
	return c.Actions.GetOrgVariable(ctx, org, name)
}

// Deprecated: use `client.Actions.GetRepoArtifact` instead.
func (c *Client) GetRepoActionArtifact(ctx context.Context, owner, repo string, artifactID int64) (*ActionsArtifact, *Response, error) {
	c.initServices()
	return c.Actions.GetRepoArtifact(ctx, owner, repo, artifactID)
}

// Deprecated: use `client.Actions.GetRepoArtifactArchive` instead.
func (c *Client) GetRepoActionArtifactArchive(ctx context.Context, owner, repo string, artifactID int64) ([]byte, *Response, error) {
	c.initServices()
	return c.Actions.GetRepoArtifactArchive(ctx, owner, repo, artifactID)
}

// Deprecated: use `client.Actions.GetRepoArtifactArchiveReader` instead.
func (c *Client) GetRepoActionArtifactArchiveReader(ctx context.Context, owner, repo string, artifactID int64) (io.ReadCloser, *Response, error) {
	c.initServices()
	return c.Actions.GetRepoArtifactArchiveReader(ctx, owner, repo, artifactID)
}

// Deprecated: use `client.Actions.GetRepoRun` instead.
func (c *Client) GetRepoActionRun(ctx context.Context, owner, repo string, runID int64) (*ActionsWorkflowRun, *Response, error) {
	c.initServices()
	return c.Actions.GetRepoRun(ctx, owner, repo, runID)
}

// Deprecated: use `client.Actions.GetRepoRunJob` instead.
func (c *Client) GetRepoActionJob(ctx context.Context, owner, repo string, jobID int64) (*ActionsWorkflowJob, *Response, error) {
	c.initServices()
	return c.Actions.GetRepoRunJob(ctx, owner, repo, jobID)
}

// Deprecated: use `client.Actions.GetRepoRunJobLogs` instead.
func (c *Client) GetRepoActionJobLogs(ctx context.Context, owner, repo string, jobID int64) ([]byte, *Response, error) {
	c.initServices()
	return c.Actions.GetRepoRunJobLogs(ctx, owner, repo, jobID)
}

// Deprecated: use `client.Actions.GetRepoRunner` instead.
func (c *Client) GetRepoActionRunner(ctx context.Context, owner, repo string, runnerID int64) (*ActionsRunner, *Response, error) {
	c.initServices()
	return c.Actions.GetRepoRunner(ctx, owner, repo, runnerID)
}

// Deprecated: use `client.Actions.GetRepoVariable` instead.
func (c *Client) GetRepoActionVariable(ctx context.Context, user, repo, variableName string) (*RepoActionsVariable, *Response, error) {
	c.initServices()
	return c.Actions.GetRepoVariable(ctx, user, repo, variableName)
}

// Deprecated: use `client.Actions.GetRepoWorkflow` instead.
func (c *Client) GetRepoActionWorkflow(ctx context.Context, owner, repo, workflowID string) (*ActionsWorkflow, *Response, error) {
	c.initServices()
	return c.Actions.GetRepoWorkflow(ctx, owner, repo, workflowID)
}

// Deprecated: use `client.Actions.GetUserRunner` instead.
func (c *Client) GetUserActionRunner(ctx context.Context, runnerID int64) (*ActionsRunner, *Response, error) {
	c.initServices()
	return c.Actions.GetUserRunner(ctx, runnerID)
}

// Deprecated: use `client.Actions.GetUserVariable` instead.
func (c *Client) GetUserActionVariable(ctx context.Context, variableName string) (*ActionsVariable, *Response, error) {
	c.initServices()
	return c.Actions.GetUserVariable(ctx, variableName)
}

// Deprecated: use `client.Actions.ListGlobalRunners` instead.
func (c *Client) ListAdminActionRunners(ctx context.Context, opt ListActionsRunnersOptions) (*ActionsRunnersResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListGlobalRunners(ctx, opt)
}

// Deprecated: use `client.Actions.ListOrgRunJobs` instead.
func (c *Client) ListOrgActionJobs(ctx context.Context, org string, opt ListRepoActionsJobsOptions) (*ActionsWorkflowJobsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListOrgRunJobs(ctx, org, opt)
}

// Deprecated: use `client.Actions.ListOrgRunners` instead.
func (c *Client) ListOrgActionRunners(ctx context.Context, org string, opt ListActionsRunnersOptions) (*ActionsRunnersResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListOrgRunners(ctx, org, opt)
}

// Deprecated: use `client.Actions.ListOrgRuns` instead.
func (c *Client) ListOrgActionRuns(ctx context.Context, org string, opt ListRepoActionsRunsOptions) (*ActionsWorkflowRunsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListOrgRuns(ctx, org, opt)
}

// Deprecated: use `client.Actions.ListOrgSecrets` instead.
func (c *Client) ListOrgActionSecret(ctx context.Context, org string, opt ListOrgActionsSecretOption) ([]*Secret, *Response, error) {
	c.initServices()
	return c.Actions.ListOrgSecrets(ctx, org, opt)
}

// Deprecated: use `client.Actions.ListOrgVariables` instead.
func (c *Client) ListOrgActionVariable(ctx context.Context, org string, opt ListOrgActionsVariableOption) ([]*ActionsVariable, *Response, error) {
	c.initServices()
	return c.Actions.ListOrgVariables(ctx, org, opt)
}

// Deprecated: use `client.Actions.ListRepoArtifacts` instead.
func (c *Client) ListRepoActionArtifacts(ctx context.Context, owner, repo string, opt ListActionsArtifactsOptions) (*ActionsArtifactsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoArtifacts(ctx, owner, repo, opt)
}

// Deprecated: use `client.Actions.ListRepoJobsByRun` instead.
func (c *Client) ListRepoActionRunJobs(ctx context.Context, owner, repo string, runID int64, opt ListRepoActionsJobsOptions) (*ActionsWorkflowJobsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoJobsByRun(ctx, owner, repo, runID, opt)
}

// Deprecated: use `client.Actions.ListRepoRunArtifacts` instead.
func (c *Client) ListRepoActionRunArtifacts(ctx context.Context, owner, repo string, runID int64, opt ListActionsArtifactsOptions) (*ActionsArtifactsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoRunArtifacts(ctx, owner, repo, runID, opt)
}

// Deprecated: use `client.Actions.ListRepoRunJobs` instead.
func (c *Client) ListRepoActionJobs(ctx context.Context, owner, repo string, opt ListRepoActionsJobsOptions) (*ActionsWorkflowJobsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoRunJobs(ctx, owner, repo, opt)
}

// Deprecated: use `client.Actions.ListRepoRunners` instead.
func (c *Client) ListRepoActionRunners(ctx context.Context, owner, repo string, opt ListActionsRunnersOptions) (*ActionsRunnersResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoRunners(ctx, owner, repo, opt)
}

// Deprecated: use `client.Actions.ListRepoRuns` instead.
func (c *Client) ListRepoActionRuns(ctx context.Context, owner, repo string, opt ListRepoActionsRunsOptions) (*ActionsWorkflowRunsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoRuns(ctx, owner, repo, opt)
}

// Deprecated: use `client.Actions.ListRepoSecrets` instead.
func (c *Client) ListRepoActionSecret(ctx context.Context, user, repo string, opt ListRepoActionsSecretOption) ([]*Secret, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoSecrets(ctx, user, repo, opt)
}

// Deprecated: use `client.Actions.ListRepoTasks` instead.
func (c *Client) ListRepoActionTasks(ctx context.Context, owner, repo string, opt ListOptions) (*ActionsTaskResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoTasks(ctx, owner, repo, opt)
}

// Deprecated: use `client.Actions.ListRepoVariables` instead.
func (c *Client) ListRepoActionVariable(ctx context.Context, user, repo string, opt ListRepoActionsVariableOption) ([]*RepoActionsVariable, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoVariables(ctx, user, repo, opt)
}

// Deprecated: use `client.Actions.ListRepoWorkflows` instead.
func (c *Client) ListRepoActionWorkflows(ctx context.Context, owner, repo string) (*ActionsWorkflowResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRepoWorkflows(ctx, owner, repo)
}

// Deprecated: use `client.Actions.ListRunJobs` instead.
func (c *Client) ListAdminActionJobs(ctx context.Context, opt ListRepoActionsJobsOptions) (*ActionsWorkflowJobsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRunJobs(ctx, opt)
}

// Deprecated: use `client.Actions.ListRuns` instead.
func (c *Client) ListAdminActionRuns(ctx context.Context, opt ListRepoActionsRunsOptions) (*ActionsWorkflowRunsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListRuns(ctx, opt)
}

// Deprecated: use `client.Actions.ListUserRunJobs` instead.
func (c *Client) ListUserActionJobs(ctx context.Context, opt ListRepoActionsJobsOptions) (*ActionsWorkflowJobsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListUserRunJobs(ctx, opt)
}

// Deprecated: use `client.Actions.ListUserRunners` instead.
func (c *Client) ListUserActionRunners(ctx context.Context, opt ListActionsRunnersOptions) (*ActionsRunnersResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListUserRunners(ctx, opt)
}

// Deprecated: use `client.Actions.ListUserRuns` instead.
func (c *Client) ListUserActionRuns(ctx context.Context, opt ListRepoActionsRunsOptions) (*ActionsWorkflowRunsResponse, *Response, error) {
	c.initServices()
	return c.Actions.ListUserRuns(ctx, opt)
}

// Deprecated: use `client.Actions.ListUserVariables` instead.
func (c *Client) ListUserActionVariable(ctx context.Context, opt ListOptions) ([]*ActionsVariable, *Response, error) {
	c.initServices()
	return c.Actions.ListUserVariables(ctx, opt)
}

// Deprecated: use `client.Actions.RerunRepoRun` instead.
func (c *Client) RerunRepoActionRun(ctx context.Context, owner, repo string, runID int64) (*ActionsWorkflowRun, *Response, error) {
	c.initServices()
	return c.Actions.RerunRepoRun(ctx, owner, repo, runID)
}

// Deprecated: use `client.Actions.RerunRepoRunFailedJobs` instead.
func (c *Client) RerunRepoActionRunFailedJobs(ctx context.Context, owner, repo string, runID int64) (*Response, error) {
	c.initServices()
	return c.Actions.RerunRepoRunFailedJobs(ctx, owner, repo, runID)
}

// Deprecated: use `client.Actions.RerunRepoRunJob` instead.
func (c *Client) RerunRepoActionJob(ctx context.Context, owner, repo string, runID, jobID int64) (*ActionsWorkflowJob, *Response, error) {
	c.initServices()
	return c.Actions.RerunRepoRunJob(ctx, owner, repo, runID, jobID)
}

// Deprecated: use `client.Actions.UpdateGlobalRunner` instead.
func (c *Client) UpdateAdminActionRunner(ctx context.Context, runnerID int64, opt EditActionsRunnerOption) (*ActionsRunner, *Response, error) {
	c.initServices()
	return c.Actions.UpdateGlobalRunner(ctx, runnerID, opt)
}

// Deprecated: use `client.Actions.UpdateOrgRunner` instead.
func (c *Client) UpdateOrgActionRunner(ctx context.Context, org string, runnerID int64, opt EditActionsRunnerOption) (*ActionsRunner, *Response, error) {
	c.initServices()
	return c.Actions.UpdateOrgRunner(ctx, org, runnerID, opt)
}

// Deprecated: use `client.Actions.UpdateOrgVariable` instead.
func (c *Client) UpdateOrgActionVariable(ctx context.Context, org, name string, opt UpdateActionsVariableOption) (*Response, error) {
	c.initServices()
	return c.Actions.UpdateOrgVariable(ctx, org, name, opt)
}

// Deprecated: use `client.Actions.UpdateRepoRunner` instead.
func (c *Client) UpdateRepoActionRunner(ctx context.Context, owner, repo string, runnerID int64, opt EditActionsRunnerOption) (*ActionsRunner, *Response, error) {
	c.initServices()
	return c.Actions.UpdateRepoRunner(ctx, owner, repo, runnerID, opt)
}

// Deprecated: use `client.Actions.UpdateRepoVariable` instead.
func (c *Client) UpdateRepoActionVariable(ctx context.Context, user, repo, variableName, value string) (*Response, error) {
	c.initServices()
	return c.Actions.UpdateRepoVariable(ctx, user, repo, variableName, value)
}

// Deprecated: use `client.Actions.UpdateUserRunner` instead.
func (c *Client) UpdateUserActionRunner(ctx context.Context, runnerID int64, opt EditActionsRunnerOption) (*ActionsRunner, *Response, error) {
	c.initServices()
	return c.Actions.UpdateUserRunner(ctx, runnerID, opt)
}

// Deprecated: use `client.Actions.UpdateUserVariable` instead.
func (c *Client) UpdateUserActionVariable(ctx context.Context, variableName string, opt UpdateActionsVariableOption) (*Response, error) {
	c.initServices()
	return c.Actions.UpdateUserVariable(ctx, variableName, opt)
}

// Deprecated: use `client.Activity.GetUserHeatmap` instead.
func (c *Client) GetUserHeatmap(ctx context.Context, username string) ([]*UserHeatmapData, *Response, error) {
	c.initServices()
	return c.Activity.GetUserHeatmap(ctx, username)
}

// Deprecated: use `client.Activity.ListOrgFeeds` instead.
func (c *Client) ListOrgActivityFeeds(ctx context.Context, org string, opt ListOrgActivityFeedsOptions) ([]*Activity, *Response, error) {
	c.initServices()
	return c.Activity.ListOrgFeeds(ctx, org, opt)
}

// Deprecated: use `client.Activity.ListRepoFeeds` instead.
func (c *Client) ListRepoActivityFeeds(ctx context.Context, owner, repo string, opt ListRepoActivityFeedsOptions) ([]*Activity, *Response, error) {
	c.initServices()
	return c.Activity.ListRepoFeeds(ctx, owner, repo, opt)
}

// Deprecated: use `client.Activity.ListTeamFeeds` instead.
func (c *Client) ListTeamActivityFeeds(ctx context.Context, teamID int64, opt ListTeamActivityFeedsOptions) ([]*Activity, *Response, error) {
	c.initServices()
	return c.Activity.ListTeamFeeds(ctx, teamID, opt)
}

// Deprecated: use `client.Activity.ListUserFeeds` instead.
func (c *Client) ListUserActivityFeeds(ctx context.Context, username string, opt ListUserActivityFeedsOptions) ([]*Activity, *Response, error) {
	c.initServices()
	return c.Activity.ListUserFeeds(ctx, username, opt)
}

// Deprecated: use `client.ActivityPub.GetPerson` instead.
func (c *Client) GetActivityPubPerson(ctx context.Context, userID int64) (ActivityPub, *Response, error) {
	c.initServices()
	return c.ActivityPub.GetPerson(ctx, userID)
}

// Deprecated: use `client.ActivityPub.GetPersonResponse` instead.
func (c *Client) GetActivityPubPersonResponse(ctx context.Context, userID int64) ([]byte, *Response, error) {
	c.initServices()
	return c.ActivityPub.GetPersonResponse(ctx, userID)
}

// Deprecated: use `client.ActivityPub.SendInbox` instead.
func (c *Client) SendActivityPubInbox(ctx context.Context, userID int64, activity ActivityPub) (*Response, error) {
	c.initServices()
	return c.ActivityPub.SendInbox(ctx, userID, activity)
}

// Deprecated: use `client.Admin.AddUserBadges` instead.
func (c *Client) AddUserBadges(ctx context.Context, username string, opt UserBadgeOption) (*Response, error) {
	c.initServices()
	return c.Admin.AddUserBadges(ctx, username, opt)
}

// Deprecated: use `client.Admin.AdoptUnadoptedRepo` instead.
func (c *Client) AdoptUnadoptedRepo(ctx context.Context, owner, repo string) (*Response, error) {
	c.initServices()
	return c.Admin.AdoptUnadoptedRepo(ctx, owner, repo)
}

// Deprecated: use `client.Admin.CreateOrg` instead.
func (c *Client) AdminCreateOrg(ctx context.Context, user string, opt CreateOrgOption) (*Organization, *Response, error) {
	c.initServices()
	return c.Admin.CreateOrg(ctx, user, opt)
}

// Deprecated: use `client.Admin.CreateRepo` instead.
func (c *Client) AdminCreateRepo(ctx context.Context, user string, opt CreateRepoOption) (*Repository, *Response, error) {
	c.initServices()
	return c.Admin.CreateRepo(ctx, user, opt)
}

// Deprecated: use `client.Admin.CreateUser` instead.
func (c *Client) AdminCreateUser(ctx context.Context, opt CreateUserOption) (*User, *Response, error) {
	c.initServices()
	return c.Admin.CreateUser(ctx, opt)
}

// Deprecated: use `client.Admin.CreateUserPublicKey` instead.
func (c *Client) AdminCreateUserPublicKey(ctx context.Context, user string, opt CreateKeyOption) (*PublicKey, *Response, error) {
	c.initServices()
	return c.Admin.CreateUserPublicKey(ctx, user, opt)
}

// Deprecated: use `client.Admin.DeleteUnadoptedRepo` instead.
func (c *Client) DeleteUnadoptedRepo(ctx context.Context, owner, repo string) (*Response, error) {
	c.initServices()
	return c.Admin.DeleteUnadoptedRepo(ctx, owner, repo)
}

// Deprecated: use `client.Admin.DeleteUser` instead.
func (c *Client) AdminDeleteUser(ctx context.Context, user string) (*Response, error) {
	c.initServices()
	return c.Admin.DeleteUser(ctx, user)
}

// Deprecated: use `client.Admin.DeleteUserBadge` instead.
func (c *Client) DeleteUserBadge(ctx context.Context, username string, opt UserBadgeOption) (*Response, error) {
	c.initServices()
	return c.Admin.DeleteUserBadge(ctx, username, opt)
}

// Deprecated: use `client.Admin.DeleteUserPublicKey` instead.
func (c *Client) AdminDeleteUserPublicKey(ctx context.Context, user string, keyID int) (*Response, error) {
	c.initServices()
	return c.Admin.DeleteUserPublicKey(ctx, user, keyID)
}

// Deprecated: use `client.Admin.EditUser` instead.
func (c *Client) AdminEditUser(ctx context.Context, user string, opt EditUserOption) (*Response, error) {
	c.initServices()
	return c.Admin.EditUser(ctx, user, opt)
}

// Deprecated: use `client.Admin.ListCronTasks` instead.
func (c *Client) ListCronTasks(ctx context.Context, opt ListCronTaskOptions) ([]*CronTask, *Response, error) {
	c.initServices()
	return c.Admin.ListCronTasks(ctx, opt)
}

// Deprecated: use `client.Admin.ListEmails` instead.
func (c *Client) ListAdminEmails(ctx context.Context, opt ListAdminEmailsOptions) ([]*Email, *Response, error) {
	c.initServices()
	return c.Admin.ListEmails(ctx, opt)
}

// Deprecated: use `client.Admin.ListOrgs` instead.
func (c *Client) AdminListOrgs(ctx context.Context, opt AdminListOrgsOptions) ([]*Organization, *Response, error) {
	c.initServices()
	return c.Admin.ListOrgs(ctx, opt)
}

// Deprecated: use `client.Admin.ListUnadoptedRepos` instead.
func (c *Client) ListUnadoptedRepos(ctx context.Context, opt ListUnadoptedReposOptions) ([]string, *Response, error) {
	c.initServices()
	return c.Admin.ListUnadoptedRepos(ctx, opt)
}

// Deprecated: use `client.Admin.ListUserBadges` instead.
func (c *Client) ListUserBadges(ctx context.Context, username string) ([]*Badge, *Response, error) {
	c.initServices()
	return c.Admin.ListUserBadges(ctx, username)
}

// Deprecated: use `client.Admin.ListUsers` instead.
func (c *Client) AdminListUsers(ctx context.Context, opt AdminListUsersOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Admin.ListUsers(ctx, opt)
}

// Deprecated: use `client.Admin.RenameUser` instead.
func (c *Client) AdminRenameUser(ctx context.Context, username string, opt RenameUserOption) (*Response, error) {
	c.initServices()
	return c.Admin.RenameUser(ctx, username, opt)
}

// Deprecated: use `client.Admin.RunCronTasks` instead.
func (c *Client) RunCronTasks(ctx context.Context, task string) (*Response, error) {
	c.initServices()
	return c.Admin.RunCronTasks(ctx, task)
}

// Deprecated: use `client.Admin.SearchEmails` instead.
func (c *Client) SearchAdminEmails(ctx context.Context, opt SearchAdminEmailsOptions) ([]*Email, *Response, error) {
	c.initServices()
	return c.Admin.SearchEmails(ctx, opt)
}

// Deprecated: use `client.Git.DeleteRepoGitHook` instead.
func (c *Client) DeleteRepoGitHook(ctx context.Context, user, repo, id string) (*Response, error) {
	c.initServices()
	return c.Git.DeleteRepoGitHook(ctx, user, repo, id)
}

// Deprecated: use `client.Git.EditRepoGitHook` instead.
func (c *Client) EditRepoGitHook(ctx context.Context, user, repo, id string, opt EditGitHookOption) (*Response, error) {
	c.initServices()
	return c.Git.EditRepoGitHook(ctx, user, repo, id, opt)
}

// Deprecated: use `client.Git.GetBlob` instead.
func (c *Client) GetBlob(ctx context.Context, user, repo, sha string) (*GitBlobResponse, *Response, error) {
	c.initServices()
	return c.Git.GetBlob(ctx, user, repo, sha)
}

// Deprecated: use `client.Git.GetRepoGitHook` instead.
func (c *Client) GetRepoGitHook(ctx context.Context, user, repo, id string) (*GitHook, *Response, error) {
	c.initServices()
	return c.Git.GetRepoGitHook(ctx, user, repo, id)
}

// Deprecated: use `client.Git.GetRepoNote` instead.
func (c *Client) GetRepoNote(ctx context.Context, owner, repo, sha string, opt GetRepoNoteOptions) (*GitNote, *Response, error) {
	c.initServices()
	return c.Git.GetRepoNote(ctx, owner, repo, sha, opt)
}

// Deprecated: use `client.Git.GetRepoRef` instead.
func (c *Client) GetRepoRef(ctx context.Context, user, repo, ref string) (*Reference, *Response, error) {
	c.initServices()
	return c.Git.GetRepoRef(ctx, user, repo, ref)
}

// Deprecated: use `client.Git.GetRepoRefs` instead.
func (c *Client) GetRepoRefs(ctx context.Context, user, repo, ref string) ([]*Reference, *Response, error) {
	c.initServices()
	return c.Git.GetRepoRefs(ctx, user, repo, ref)
}

// Deprecated: use `client.Git.GetTrees` instead.
func (c *Client) GetTrees(ctx context.Context, user, repo string, opt ListTreeOptions) (*GitTreeResponse, *Response, error) {
	c.initServices()
	return c.Git.GetTrees(ctx, user, repo, opt)
}

// Deprecated: use `client.Git.ListAllGitRefs` instead.
func (c *Client) ListAllGitRefs(ctx context.Context, owner, repo string) ([]*Reference, *Response, error) {
	c.initServices()
	return c.Git.ListAllGitRefs(ctx, owner, repo)
}

// Deprecated: use `client.Git.ListRepoGitHooks` instead.
func (c *Client) ListRepoGitHooks(ctx context.Context, user, repo string, opt ListRepoGitHooksOptions) ([]*GitHook, *Response, error) {
	c.initServices()
	return c.Git.ListRepoGitHooks(ctx, user, repo, opt)
}

// Deprecated: use `client.Hooks.CreateGlobalHook` instead.
func (c *Client) CreateAdminHook(ctx context.Context, opt CreateHookOption) (*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.CreateGlobalHook(ctx, opt)
}

// Deprecated: use `client.Hooks.CreateMyHook` instead.
func (c *Client) CreateMyHook(ctx context.Context, opt CreateHookOption) (*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.CreateMyHook(ctx, opt)
}

// Deprecated: use `client.Hooks.CreateOrgHook` instead.
func (c *Client) CreateOrgHook(ctx context.Context, org string, opt CreateHookOption) (*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.CreateOrgHook(ctx, org, opt)
}

// Deprecated: use `client.Hooks.CreateRepoHook` instead.
func (c *Client) CreateRepoHook(ctx context.Context, user, repo string, opt CreateHookOption) (*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.CreateRepoHook(ctx, user, repo, opt)
}

// Deprecated: use `client.Hooks.DeleteGlobalHook` instead.
func (c *Client) DeleteAdminHook(ctx context.Context, id int64) (*Response, error) {
	c.initServices()
	return c.Hooks.DeleteGlobalHook(ctx, id)
}

// Deprecated: use `client.Hooks.DeleteMyHook` instead.
func (c *Client) DeleteMyHook(ctx context.Context, id int64) (*Response, error) {
	c.initServices()
	return c.Hooks.DeleteMyHook(ctx, id)
}

// Deprecated: use `client.Hooks.DeleteOrgHook` instead.
func (c *Client) DeleteOrgHook(ctx context.Context, org string, id int64) (*Response, error) {
	c.initServices()
	return c.Hooks.DeleteOrgHook(ctx, org, id)
}

// Deprecated: use `client.Hooks.DeleteRepoHook` instead.
func (c *Client) DeleteRepoHook(ctx context.Context, user, repo string, id int64) (*Response, error) {
	c.initServices()
	return c.Hooks.DeleteRepoHook(ctx, user, repo, id)
}

// Deprecated: use `client.Hooks.EditGlobalHook` instead.
func (c *Client) EditAdminHook(ctx context.Context, id int64, opt EditHookOption) (*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.EditGlobalHook(ctx, id, opt)
}

// Deprecated: use `client.Hooks.EditMyHook` instead.
func (c *Client) EditMyHook(ctx context.Context, id int64, opt EditHookOption) (*Response, error) {
	c.initServices()
	return c.Hooks.EditMyHook(ctx, id, opt)
}

// Deprecated: use `client.Hooks.EditOrgHook` instead.
func (c *Client) EditOrgHook(ctx context.Context, org string, id int64, opt EditHookOption) (*Response, error) {
	c.initServices()
	return c.Hooks.EditOrgHook(ctx, org, id, opt)
}

// Deprecated: use `client.Hooks.EditRepoHook` instead.
func (c *Client) EditRepoHook(ctx context.Context, user, repo string, id int64, opt EditHookOption) (*Response, error) {
	c.initServices()
	return c.Hooks.EditRepoHook(ctx, user, repo, id, opt)
}

// Deprecated: use `client.Hooks.GetGlobalHook` instead.
func (c *Client) GetAdminHook(ctx context.Context, id int64) (*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.GetGlobalHook(ctx, id)
}

// Deprecated: use `client.Hooks.GetMyHook` instead.
func (c *Client) GetMyHook(ctx context.Context, id int64) (*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.GetMyHook(ctx, id)
}

// Deprecated: use `client.Hooks.GetOrgHook` instead.
func (c *Client) GetOrgHook(ctx context.Context, org string, id int64) (*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.GetOrgHook(ctx, org, id)
}

// Deprecated: use `client.Hooks.GetRepoHook` instead.
func (c *Client) GetRepoHook(ctx context.Context, user, repo string, id int64) (*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.GetRepoHook(ctx, user, repo, id)
}

// Deprecated: use `client.Hooks.ListGlobalHooks` instead.
func (c *Client) ListAdminHooks(ctx context.Context, opt ListGlobalHooksOptions) ([]*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.ListGlobalHooks(ctx, opt)
}

// Deprecated: use `client.Hooks.ListMyHooks` instead.
func (c *Client) ListMyHooks(ctx context.Context, opt ListHooksOptions) ([]*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.ListMyHooks(ctx, opt)
}

// Deprecated: use `client.Hooks.ListOrgHooks` instead.
func (c *Client) ListOrgHooks(ctx context.Context, org string, opt ListHooksOptions) ([]*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.ListOrgHooks(ctx, org, opt)
}

// Deprecated: use `client.Hooks.ListRepoHooks` instead.
func (c *Client) ListRepoHooks(ctx context.Context, user, repo string, opt ListHooksOptions) ([]*Hook, *Response, error) {
	c.initServices()
	return c.Hooks.ListRepoHooks(ctx, user, repo, opt)
}

// Deprecated: use `client.Hooks.TestWebhook` instead.
func (c *Client) TestWebhook(ctx context.Context, owner, repo string, hookID int64, ref string) (*Response, error) {
	c.initServices()
	return c.Hooks.TestWebhook(ctx, owner, repo, hookID, ref)
}

// Deprecated: use `client.Issues.AddIssueLabels` instead.
func (c *Client) AddIssueLabels(ctx context.Context, owner, repo string, index int64, opt IssueLabelsOption) ([]*Label, *Response, error) {
	c.initServices()
	return c.Issues.AddIssueLabels(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.AddIssueSubscription` instead.
func (c *Client) AddIssueSubscription(ctx context.Context, owner, repo string, index int64, user string) (*Response, error) {
	c.initServices()
	return c.Issues.AddIssueSubscription(ctx, owner, repo, index, user)
}

// Deprecated: use `client.Issues.AddTime` instead.
func (c *Client) AddTime(ctx context.Context, owner, repo string, index int64, opt AddTimeOption) (*TrackedTime, *Response, error) {
	c.initServices()
	return c.Issues.AddTime(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.CheckIssueSubscription` instead.
func (c *Client) CheckIssueSubscription(ctx context.Context, owner, repo string, index int64) (*WatchInfo, *Response, error) {
	c.initServices()
	return c.Issues.CheckIssueSubscription(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.ClearIssueLabels` instead.
func (c *Client) ClearIssueLabels(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.ClearIssueLabels(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.CreateIssue` instead.
func (c *Client) CreateIssue(ctx context.Context, owner, repo string, opt CreateIssueOption) (*Issue, *Response, error) {
	c.initServices()
	return c.Issues.CreateIssue(ctx, owner, repo, opt)
}

// Deprecated: use `client.Issues.CreateIssueAttachment` instead.
func (c *Client) CreateIssueAttachment(ctx context.Context, owner, repo string, index int64, file io.Reader, filename string) (*Attachment, *Response, error) {
	c.initServices()
	return c.Issues.CreateIssueAttachment(ctx, owner, repo, index, file, filename)
}

// Deprecated: use `client.Issues.CreateIssueBlocking` instead.
func (c *Client) CreateIssueBlocking(ctx context.Context, owner, repo string, index int64, opt IssueMeta) (*Issue, *Response, error) {
	c.initServices()
	return c.Issues.CreateIssueBlocking(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.CreateIssueComment` instead.
func (c *Client) CreateIssueComment(ctx context.Context, owner, repo string, index int64, opt CreateIssueCommentOption) (*Comment, *Response, error) {
	c.initServices()
	return c.Issues.CreateIssueComment(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.CreateIssueCommentAttachment` instead.
func (c *Client) CreateIssueCommentAttachment(ctx context.Context, owner, repo string, commentID int64, file io.Reader, filename string) (*Attachment, *Response, error) {
	c.initServices()
	return c.Issues.CreateIssueCommentAttachment(ctx, owner, repo, commentID, file, filename)
}

// Deprecated: use `client.Issues.CreateIssueDependency` instead.
func (c *Client) CreateIssueDependency(ctx context.Context, owner, repo string, index int64, opt IssueMeta) (*Issue, *Response, error) {
	c.initServices()
	return c.Issues.CreateIssueDependency(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.DeleteIssue` instead.
func (c *Client) DeleteIssue(ctx context.Context, user, repo string, id int64) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteIssue(ctx, user, repo, id)
}

// Deprecated: use `client.Issues.DeleteIssueAttachment` instead.
func (c *Client) DeleteIssueAttachment(ctx context.Context, owner, repo string, index, attachmentID int64) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteIssueAttachment(ctx, owner, repo, index, attachmentID)
}

// Deprecated: use `client.Issues.DeleteIssueComment` instead.
func (c *Client) DeleteIssueComment(ctx context.Context, owner, repo string, commentID int64) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteIssueComment(ctx, owner, repo, commentID)
}

// Deprecated: use `client.Issues.DeleteIssueCommentAttachment` instead.
func (c *Client) DeleteIssueCommentAttachment(ctx context.Context, owner, repo string, commentID, attachmentID int64) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteIssueCommentAttachment(ctx, owner, repo, commentID, attachmentID)
}

// Deprecated: use `client.Issues.DeleteIssueCommentReaction` instead.
func (c *Client) DeleteIssueCommentReaction(ctx context.Context, owner, repo string, commentID int64, reaction string) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteIssueCommentReaction(ctx, owner, repo, commentID, reaction)
}

// Deprecated: use `client.Issues.DeleteIssueLabel` instead.
func (c *Client) DeleteIssueLabel(ctx context.Context, owner, repo string, index, label int64) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteIssueLabel(ctx, owner, repo, index, label)
}

// Deprecated: use `client.Issues.DeleteIssueReaction` instead.
func (c *Client) DeleteIssueReaction(ctx context.Context, owner, repo string, index int64, reaction string) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteIssueReaction(ctx, owner, repo, index, reaction)
}

// Deprecated: use `client.Issues.DeleteIssueStopwatch` instead.
func (c *Client) DeleteIssueStopwatch(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteIssueStopwatch(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.DeleteIssueSubscription` instead.
func (c *Client) DeleteIssueSubscription(ctx context.Context, owner, repo string, index int64, user string) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteIssueSubscription(ctx, owner, repo, index, user)
}

// Deprecated: use `client.Issues.DeleteTime` instead.
func (c *Client) DeleteTime(ctx context.Context, owner, repo string, index, timeID int64) (*Response, error) {
	c.initServices()
	return c.Issues.DeleteTime(ctx, owner, repo, index, timeID)
}

// Deprecated: use `client.Issues.EditIssue` instead.
func (c *Client) EditIssue(ctx context.Context, owner, repo string, index int64, opt EditIssueOption) (*Issue, *Response, error) {
	c.initServices()
	return c.Issues.EditIssue(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.EditIssueAttachment` instead.
func (c *Client) EditIssueAttachment(ctx context.Context, owner, repo string, index, attachmentID int64, form EditAttachmentOptions) (*Attachment, *Response, error) {
	c.initServices()
	return c.Issues.EditIssueAttachment(ctx, owner, repo, index, attachmentID, form)
}

// Deprecated: use `client.Issues.EditIssueComment` instead.
func (c *Client) EditIssueComment(ctx context.Context, owner, repo string, commentID int64, opt EditIssueCommentOption) (*Comment, *Response, error) {
	c.initServices()
	return c.Issues.EditIssueComment(ctx, owner, repo, commentID, opt)
}

// Deprecated: use `client.Issues.EditIssueCommentAttachment` instead.
func (c *Client) EditIssueCommentAttachment(ctx context.Context, owner, repo string, commentID, attachmentID int64, form EditAttachmentOptions) (*Attachment, *Response, error) {
	c.initServices()
	return c.Issues.EditIssueCommentAttachment(ctx, owner, repo, commentID, attachmentID, form)
}

// Deprecated: use `client.Issues.GetIssue` instead.
func (c *Client) GetIssue(ctx context.Context, owner, repo string, index int64) (*Issue, *Response, error) {
	c.initServices()
	return c.Issues.GetIssue(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.GetIssueAttachment` instead.
func (c *Client) GetIssueAttachment(ctx context.Context, owner, repo string, index, attachmentID int64) (*Attachment, *Response, error) {
	c.initServices()
	return c.Issues.GetIssueAttachment(ctx, owner, repo, index, attachmentID)
}

// Deprecated: use `client.Issues.GetIssueComment` instead.
func (c *Client) GetIssueComment(ctx context.Context, owner, repo string, id int64) (*Comment, *Response, error) {
	c.initServices()
	return c.Issues.GetIssueComment(ctx, owner, repo, id)
}

// Deprecated: use `client.Issues.GetIssueCommentAttachment` instead.
func (c *Client) GetIssueCommentAttachment(ctx context.Context, owner, repo string, commentID, attachmentID int64) (*Attachment, *Response, error) {
	c.initServices()
	return c.Issues.GetIssueCommentAttachment(ctx, owner, repo, commentID, attachmentID)
}

// Deprecated: use `client.Issues.GetIssueCommentReactions` instead.
func (c *Client) GetIssueCommentReactions(ctx context.Context, owner, repo string, commentID int64) ([]*Reaction, *Response, error) {
	c.initServices()
	return c.Issues.GetIssueCommentReactions(ctx, owner, repo, commentID)
}

// Deprecated: use `client.Issues.GetIssueLabels` instead.
func (c *Client) GetIssueLabels(ctx context.Context, owner, repo string, index int64, opts ListLabelsOptions) ([]*Label, *Response, error) {
	c.initServices()
	return c.Issues.GetIssueLabels(ctx, owner, repo, index, opts)
}

// Deprecated: use `client.Issues.GetIssueTemplates` instead.
func (c *Client) GetIssueTemplates(ctx context.Context, owner, repo string) ([]*IssueTemplate, *Response, error) {
	c.initServices()
	return c.Issues.GetIssueTemplates(ctx, owner, repo)
}

// Deprecated: use `client.Issues.GetUserTrackedTimes` instead.
func (c *Client) GetUserTrackedTimes(ctx context.Context, owner, repo, user string) ([]*TrackedTime, *Response, error) {
	c.initServices()
	return c.Issues.GetUserTrackedTimes(ctx, owner, repo, user)
}

// Deprecated: use `client.Issues.IssueSubscribe` instead.
func (c *Client) IssueSubscribe(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.IssueSubscribe(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.IssueUnSubscribe` instead.
func (c *Client) IssueUnSubscribe(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.IssueUnSubscribe(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.ListIssueAttachments` instead.
func (c *Client) ListIssueAttachments(ctx context.Context, owner, repo string, index int64) ([]*Attachment, *Response, error) {
	c.initServices()
	return c.Issues.ListIssueAttachments(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.ListIssueBlocks` instead.
func (c *Client) ListIssueBlocks(ctx context.Context, owner, repo string, index int64, opt ListIssueBlocksOptions) ([]*Issue, *Response, error) {
	c.initServices()
	return c.Issues.ListIssueBlocks(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.ListIssueCommentAttachments` instead.
func (c *Client) ListIssueCommentAttachments(ctx context.Context, owner, repo string, commentID int64) ([]*Attachment, *Response, error) {
	c.initServices()
	return c.Issues.ListIssueCommentAttachments(ctx, owner, repo, commentID)
}

// Deprecated: use `client.Issues.ListIssueComments` instead.
func (c *Client) ListIssueComments(ctx context.Context, owner, repo string, index int64, opt ListIssueCommentOptions) ([]*Comment, *Response, error) {
	c.initServices()
	return c.Issues.ListIssueComments(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.ListIssueDependencies` instead.
func (c *Client) ListIssueDependencies(ctx context.Context, owner, repo string, index int64, opt ListIssueDependenciesOptions) ([]*Issue, *Response, error) {
	c.initServices()
	return c.Issues.ListIssueDependencies(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.ListIssueReactions` instead.
func (c *Client) ListIssueReactions(ctx context.Context, owner, repo string, index int64, opt ListIssueReactionsOptions) ([]*Reaction, *Response, error) {
	c.initServices()
	return c.Issues.ListIssueReactions(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.ListIssueSubscribers` instead.
func (c *Client) ListIssueSubscribers(ctx context.Context, owner, repo string, index int64, opt ListIssueSubscribersOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Issues.ListIssueSubscribers(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.ListIssueTimeline` instead.
func (c *Client) ListIssueTimeline(ctx context.Context, owner, repo string, index int64, opt ListIssueCommentOptions) ([]*TimelineComment, *Response, error) {
	c.initServices()
	return c.Issues.ListIssueTimeline(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.ListIssueTrackedTimes` instead.
func (c *Client) ListIssueTrackedTimes(ctx context.Context, owner, repo string, index int64, opt ListTrackedTimesOptions) ([]*TrackedTime, *Response, error) {
	c.initServices()
	return c.Issues.ListIssueTrackedTimes(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.ListIssues` instead.
func (c *Client) ListIssues(ctx context.Context, opt ListIssueOption) ([]*Issue, *Response, error) {
	c.initServices()
	return c.Issues.ListIssues(ctx, opt)
}

// Deprecated: use `client.Issues.ListMyStopwatches` instead.
func (c *Client) ListMyStopwatches(ctx context.Context, opt ListStopwatchesOptions) ([]*StopWatch, *Response, error) {
	c.initServices()
	return c.Issues.ListMyStopwatches(ctx, opt)
}

// Deprecated: use `client.Issues.ListMyTrackedTimes` instead.
func (c *Client) ListMyTrackedTimes(ctx context.Context, opt ListTrackedTimesOptions) ([]*TrackedTime, *Response, error) {
	c.initServices()
	return c.Issues.ListMyTrackedTimes(ctx, opt)
}

// Deprecated: use `client.Issues.ListRepoIssueComments` instead.
func (c *Client) ListRepoIssueComments(ctx context.Context, owner, repo string, opt ListIssueCommentOptions) ([]*Comment, *Response, error) {
	c.initServices()
	return c.Issues.ListRepoIssueComments(ctx, owner, repo, opt)
}

// Deprecated: use `client.Issues.ListRepoIssues` instead.
func (c *Client) ListRepoIssues(ctx context.Context, owner, repo string, opt ListIssueOption) ([]*Issue, *Response, error) {
	c.initServices()
	return c.Issues.ListRepoIssues(ctx, owner, repo, opt)
}

// Deprecated: use `client.Issues.ListRepoPinnedIssues` instead.
func (c *Client) ListRepoPinnedIssues(ctx context.Context, owner, repo string) ([]*Issue, *Response, error) {
	c.initServices()
	return c.Issues.ListRepoPinnedIssues(ctx, owner, repo)
}

// Deprecated: use `client.Issues.ListRepoTrackedTimes` instead.
func (c *Client) ListRepoTrackedTimes(ctx context.Context, owner, repo string, opt ListTrackedTimesOptions) ([]*TrackedTime, *Response, error) {
	c.initServices()
	return c.Issues.ListRepoTrackedTimes(ctx, owner, repo, opt)
}

// Deprecated: use `client.Issues.LockIssue` instead.
func (c *Client) LockIssue(ctx context.Context, owner, repo string, index int64, opt LockIssueOption) (*Response, error) {
	c.initServices()
	return c.Issues.LockIssue(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.MoveIssuePin` instead.
func (c *Client) MoveIssuePin(ctx context.Context, owner, repo string, index, position int64) (*Response, error) {
	c.initServices()
	return c.Issues.MoveIssuePin(ctx, owner, repo, index, position)
}

// Deprecated: use `client.Issues.PinIssue` instead.
func (c *Client) PinIssue(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.PinIssue(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.PostIssueCommentReaction` instead.
func (c *Client) PostIssueCommentReaction(ctx context.Context, owner, repo string, commentID int64, reaction string) (*Reaction, *Response, error) {
	c.initServices()
	return c.Issues.PostIssueCommentReaction(ctx, owner, repo, commentID, reaction)
}

// Deprecated: use `client.Issues.PostIssueReaction` instead.
func (c *Client) PostIssueReaction(ctx context.Context, owner, repo string, index int64, reaction string) (*Reaction, *Response, error) {
	c.initServices()
	return c.Issues.PostIssueReaction(ctx, owner, repo, index, reaction)
}

// Deprecated: use `client.Issues.RemoveIssueBlocking` instead.
func (c *Client) RemoveIssueBlocking(ctx context.Context, owner, repo string, index int64, opt IssueMeta) (*Issue, *Response, error) {
	c.initServices()
	return c.Issues.RemoveIssueBlocking(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.RemoveIssueDependency` instead.
func (c *Client) RemoveIssueDependency(ctx context.Context, owner, repo string, index int64, opt IssueMeta) (*Issue, *Response, error) {
	c.initServices()
	return c.Issues.RemoveIssueDependency(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.ReplaceIssueLabels` instead.
func (c *Client) ReplaceIssueLabels(ctx context.Context, owner, repo string, index int64, opt IssueLabelsOption) ([]*Label, *Response, error) {
	c.initServices()
	return c.Issues.ReplaceIssueLabels(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Issues.ResetIssueTime` instead.
func (c *Client) ResetIssueTime(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.ResetIssueTime(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.StartIssueStopWatch` instead.
func (c *Client) StartIssueStopWatch(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.StartIssueStopWatch(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.StopIssueStopWatch` instead.
func (c *Client) StopIssueStopWatch(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.StopIssueStopWatch(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.UnlockIssue` instead.
func (c *Client) UnlockIssue(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.UnlockIssue(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.UnpinIssue` instead.
func (c *Client) UnpinIssue(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.Issues.UnpinIssue(ctx, owner, repo, index)
}

// Deprecated: use `client.Issues.UpdateIssueDeadline` instead.
func (c *Client) UpdateIssueDeadline(ctx context.Context, owner, repo string, index int64, opt EditDeadlineOption) (*Issue, *Response, error) {
	c.initServices()
	return c.Issues.UpdateIssueDeadline(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.Meta.GetNodeInfo` instead.
func (c *Client) GetNodeInfo(ctx context.Context) (*NodeInfo, *Response, error) {
	c.initServices()
	return c.Meta.GetNodeInfo(ctx)
}

// Deprecated: use `client.Meta.GetSigningKeyGPG` instead.
func (c *Client) GetSigningKeyGPG(ctx context.Context) (string, *Response, error) {
	c.initServices()
	return c.Meta.GetSigningKeyGPG(ctx)
}

// Deprecated: use `client.Meta.GetSigningKeySSH` instead.
func (c *Client) GetSigningKeySSH(ctx context.Context) (string, *Response, error) {
	c.initServices()
	return c.Meta.GetSigningKeySSH(ctx)
}

// Deprecated: use `client.Meta.ServerVersion` instead.
func (c *Client) ServerVersion(ctx context.Context) (string, *Response, error) {
	c.initServices()
	return c.Meta.ServerVersion(ctx)
}

// Deprecated: use `client.Notifications.Check` instead.
func (c *Client) CheckNotifications(ctx context.Context) (int64, *Response, error) {
	c.initServices()
	return c.Notifications.Check(ctx)
}

// Deprecated: use `client.Notifications.GetByID` instead.
func (c *Client) GetNotification(ctx context.Context, id int64) (*NotificationThread, *Response, error) {
	c.initServices()
	return c.Notifications.GetByID(ctx, id)
}

// Deprecated: use `client.Notifications.List` instead.
func (c *Client) ListNotifications(ctx context.Context, opt ListNotificationOptions) ([]*NotificationThread, *Response, error) {
	c.initServices()
	return c.Notifications.List(ctx, opt)
}

// Deprecated: use `client.Notifications.ListByRepo` instead.
func (c *Client) ListRepoNotifications(ctx context.Context, owner, repo string, opt ListNotificationOptions) ([]*NotificationThread, *Response, error) {
	c.initServices()
	return c.Notifications.ListByRepo(ctx, owner, repo, opt)
}

// Deprecated: use `client.Notifications.MarkRead` instead.
func (c *Client) ReadNotifications(ctx context.Context, opt MarkNotificationOptions) ([]*NotificationThread, *Response, error) {
	c.initServices()
	return c.Notifications.MarkRead(ctx, opt)
}

// Deprecated: use `client.Notifications.MarkReadByID` instead.
func (c *Client) ReadNotification(ctx context.Context, id int64, status ...NotificationStatus) (*NotificationThread, *Response, error) {
	c.initServices()
	return c.Notifications.MarkReadByID(ctx, id, status...)
}

// Deprecated: use `client.Notifications.MarkReadByRepo` instead.
func (c *Client) ReadRepoNotifications(ctx context.Context, owner, repo string, opt MarkNotificationOptions) ([]*NotificationThread, *Response, error) {
	c.initServices()
	return c.Notifications.MarkReadByRepo(ctx, owner, repo, opt)
}

// Deprecated: use `client.OAuth2.CreateApplication` instead.
func (c *Client) CreateOauth2(ctx context.Context, opt CreateApplicationOption) (*OAuth2Application, *Response, error) {
	c.initServices()
	return c.OAuth2.CreateApplication(ctx, opt)
}

// Deprecated: use `client.OAuth2.DeleteApplication` instead.
func (c *Client) DeleteOauth2(ctx context.Context, oauth2id int64) (*Response, error) {
	c.initServices()
	return c.OAuth2.DeleteApplication(ctx, oauth2id)
}

// Deprecated: use `client.OAuth2.GetApplication` instead.
func (c *Client) GetOauth2(ctx context.Context, oauth2id int64) (*OAuth2Application, *Response, error) {
	c.initServices()
	return c.OAuth2.GetApplication(ctx, oauth2id)
}

// Deprecated: use `client.OAuth2.ListApplications` instead.
func (c *Client) ListOauth2(ctx context.Context, opt ListApplicationsOptions) ([]*OAuth2Application, *Response, error) {
	c.initServices()
	return c.OAuth2.ListApplications(ctx, opt)
}

// Deprecated: use `client.OAuth2.UpdateApplication` instead.
func (c *Client) UpdateOauth2(ctx context.Context, oauth2id int64, opt CreateApplicationOption) (*OAuth2Application, *Response, error) {
	c.initServices()
	return c.OAuth2.UpdateApplication(ctx, oauth2id, opt)
}

// Deprecated: use `client.Organizations.AddTeamMember` instead.
func (c *Client) AddTeamMember(ctx context.Context, id int64, user string) (*Response, error) {
	c.initServices()
	return c.Organizations.AddTeamMember(ctx, id, user)
}

// Deprecated: use `client.Organizations.AddTeamRepository` instead.
func (c *Client) AddTeamRepository(ctx context.Context, id int64, org, repo string) (*Response, error) {
	c.initServices()
	return c.Organizations.AddTeamRepository(ctx, id, org, repo)
}

// Deprecated: use `client.Organizations.BlockOrgUser` instead.
func (c *Client) BlockOrgUser(ctx context.Context, org, username string) (*Response, error) {
	c.initServices()
	return c.Organizations.BlockOrgUser(ctx, org, username)
}

// Deprecated: use `client.Organizations.CheckOrgBlock` instead.
func (c *Client) CheckOrgBlock(ctx context.Context, org, username string) (bool, *Response, error) {
	c.initServices()
	return c.Organizations.CheckOrgBlock(ctx, org, username)
}

// Deprecated: use `client.Organizations.CheckOrgMembership` instead.
func (c *Client) CheckOrgMembership(ctx context.Context, org, user string) (bool, *Response, error) {
	c.initServices()
	return c.Organizations.CheckOrgMembership(ctx, org, user)
}

// Deprecated: use `client.Organizations.CheckPublicOrgMembership` instead.
func (c *Client) CheckPublicOrgMembership(ctx context.Context, org, user string) (bool, *Response, error) {
	c.initServices()
	return c.Organizations.CheckPublicOrgMembership(ctx, org, user)
}

// Deprecated: use `client.Organizations.CreateOrg` instead.
func (c *Client) CreateOrg(ctx context.Context, opt CreateOrgOption) (*Organization, *Response, error) {
	c.initServices()
	return c.Organizations.CreateOrg(ctx, opt)
}

// Deprecated: use `client.Organizations.CreateOrgLabel` instead.
func (c *Client) CreateOrgLabel(ctx context.Context, orgName string, opt CreateOrgLabelOption) (*Label, *Response, error) {
	c.initServices()
	return c.Organizations.CreateOrgLabel(ctx, orgName, opt)
}

// Deprecated: use `client.Organizations.CreateTeam` instead.
func (c *Client) CreateTeam(ctx context.Context, org string, opt CreateTeamOption) (*Team, *Response, error) {
	c.initServices()
	return c.Organizations.CreateTeam(ctx, org, opt)
}

// Deprecated: use `client.Organizations.DeleteOrg` instead.
func (c *Client) DeleteOrg(ctx context.Context, orgname string) (*Response, error) {
	c.initServices()
	return c.Organizations.DeleteOrg(ctx, orgname)
}

// Deprecated: use `client.Organizations.DeleteOrgAvatar` instead.
func (c *Client) DeleteOrgAvatar(ctx context.Context, org string) (*Response, error) {
	c.initServices()
	return c.Organizations.DeleteOrgAvatar(ctx, org)
}

// Deprecated: use `client.Organizations.DeleteOrgLabel` instead.
func (c *Client) DeleteOrgLabel(ctx context.Context, orgName string, labelID int64) (*Response, error) {
	c.initServices()
	return c.Organizations.DeleteOrgLabel(ctx, orgName, labelID)
}

// Deprecated: use `client.Organizations.DeleteOrgMembership` instead.
func (c *Client) DeleteOrgMembership(ctx context.Context, org, user string) (*Response, error) {
	c.initServices()
	return c.Organizations.DeleteOrgMembership(ctx, org, user)
}

// Deprecated: use `client.Organizations.DeleteTeam` instead.
func (c *Client) DeleteTeam(ctx context.Context, id int64) (*Response, error) {
	c.initServices()
	return c.Organizations.DeleteTeam(ctx, id)
}

// Deprecated: use `client.Organizations.EditOrg` instead.
func (c *Client) EditOrg(ctx context.Context, orgname string, opt EditOrgOption) (*Response, error) {
	c.initServices()
	return c.Organizations.EditOrg(ctx, orgname, opt)
}

// Deprecated: use `client.Organizations.EditOrgLabel` instead.
func (c *Client) EditOrgLabel(ctx context.Context, orgName string, labelID int64, opt EditOrgLabelOption) (*Label, *Response, error) {
	c.initServices()
	return c.Organizations.EditOrgLabel(ctx, orgName, labelID, opt)
}

// Deprecated: use `client.Organizations.EditTeam` instead.
func (c *Client) EditTeam(ctx context.Context, id int64, opt EditTeamOption) (*Response, error) {
	c.initServices()
	return c.Organizations.EditTeam(ctx, id, opt)
}

// Deprecated: use `client.Organizations.GetOrg` instead.
func (c *Client) GetOrg(ctx context.Context, orgname string) (*Organization, *Response, error) {
	c.initServices()
	return c.Organizations.GetOrg(ctx, orgname)
}

// Deprecated: use `client.Organizations.GetOrgLabel` instead.
func (c *Client) GetOrgLabel(ctx context.Context, orgName string, labelID int64) (*Label, *Response, error) {
	c.initServices()
	return c.Organizations.GetOrgLabel(ctx, orgName, labelID)
}

// Deprecated: use `client.Organizations.GetOrgPermissions` instead.
func (c *Client) GetOrgPermissions(ctx context.Context, org, user string) (*OrgPermissions, *Response, error) {
	c.initServices()
	return c.Organizations.GetOrgPermissions(ctx, org, user)
}

// Deprecated: use `client.Organizations.GetTeam` instead.
func (c *Client) GetTeam(ctx context.Context, id int64) (*Team, *Response, error) {
	c.initServices()
	return c.Organizations.GetTeam(ctx, id)
}

// Deprecated: use `client.Organizations.GetTeamMember` instead.
func (c *Client) GetTeamMember(ctx context.Context, id int64, user string) (*User, *Response, error) {
	c.initServices()
	return c.Organizations.GetTeamMember(ctx, id, user)
}

// Deprecated: use `client.Organizations.GetTeamRepository` instead.
func (c *Client) GetTeamRepository(ctx context.Context, id int64, org, repo string) (*Repository, *Response, error) {
	c.initServices()
	return c.Organizations.GetTeamRepository(ctx, id, org, repo)
}

// Deprecated: use `client.Organizations.ListMyOrgs` instead.
func (c *Client) ListMyOrgs(ctx context.Context, opt ListOrgsOptions) ([]*Organization, *Response, error) {
	c.initServices()
	return c.Organizations.ListMyOrgs(ctx, opt)
}

// Deprecated: use `client.Organizations.ListMyTeams` instead.
func (c *Client) ListMyTeams(ctx context.Context, opt *ListTeamsOptions) ([]*Team, *Response, error) {
	c.initServices()
	return c.Organizations.ListMyTeams(ctx, opt)
}

// Deprecated: use `client.Organizations.ListOrgBlocks` instead.
func (c *Client) ListOrgBlocks(ctx context.Context, org string, opt ListOrgBlocksOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Organizations.ListOrgBlocks(ctx, org, opt)
}

// Deprecated: use `client.Organizations.ListOrgLabels` instead.
func (c *Client) ListOrgLabels(ctx context.Context, orgName string, opt ListOrgLabelsOptions) ([]*Label, *Response, error) {
	c.initServices()
	return c.Organizations.ListOrgLabels(ctx, orgName, opt)
}

// Deprecated: use `client.Organizations.ListOrgMembership` instead.
func (c *Client) ListOrgMembership(ctx context.Context, org string, opt ListOrgMembershipOption) ([]*User, *Response, error) {
	c.initServices()
	return c.Organizations.ListOrgMembership(ctx, org, opt)
}

// Deprecated: use `client.Organizations.ListOrgTeams` instead.
func (c *Client) ListOrgTeams(ctx context.Context, org string, opt ListTeamsOptions) ([]*Team, *Response, error) {
	c.initServices()
	return c.Organizations.ListOrgTeams(ctx, org, opt)
}

// Deprecated: use `client.Organizations.ListOrgs` instead.
func (c *Client) ListOrgs(ctx context.Context, opt ListOrgsOptions) ([]*Organization, *Response, error) {
	c.initServices()
	return c.Organizations.ListOrgs(ctx, opt)
}

// Deprecated: use `client.Organizations.ListPublicOrgMembership` instead.
func (c *Client) ListPublicOrgMembership(ctx context.Context, org string, opt ListOrgMembershipOption) ([]*User, *Response, error) {
	c.initServices()
	return c.Organizations.ListPublicOrgMembership(ctx, org, opt)
}

// Deprecated: use `client.Organizations.ListTeamMembers` instead.
func (c *Client) ListTeamMembers(ctx context.Context, id int64, opt ListTeamMembersOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Organizations.ListTeamMembers(ctx, id, opt)
}

// Deprecated: use `client.Organizations.ListTeamRepositories` instead.
func (c *Client) ListTeamRepositories(ctx context.Context, id int64, opt ListTeamRepositoriesOptions) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Organizations.ListTeamRepositories(ctx, id, opt)
}

// Deprecated: use `client.Organizations.ListUserOrgs` instead.
func (c *Client) ListUserOrgs(ctx context.Context, user string, opt ListOrgsOptions) ([]*Organization, *Response, error) {
	c.initServices()
	return c.Organizations.ListUserOrgs(ctx, user, opt)
}

// Deprecated: use `client.Organizations.RemoveTeamMember` instead.
func (c *Client) RemoveTeamMember(ctx context.Context, id int64, user string) (*Response, error) {
	c.initServices()
	return c.Organizations.RemoveTeamMember(ctx, id, user)
}

// Deprecated: use `client.Organizations.RemoveTeamRepository` instead.
func (c *Client) RemoveTeamRepository(ctx context.Context, id int64, org, repo string) (*Response, error) {
	c.initServices()
	return c.Organizations.RemoveTeamRepository(ctx, id, org, repo)
}

// Deprecated: use `client.Organizations.RenameOrg` instead.
func (c *Client) RenameOrg(ctx context.Context, org string, opt RenameOrgOption) (*Response, error) {
	c.initServices()
	return c.Organizations.RenameOrg(ctx, org, opt)
}

// Deprecated: use `client.Organizations.SearchOrgTeams` instead.
func (c *Client) SearchOrgTeams(ctx context.Context, org string, opt *SearchTeamsOptions) ([]*Team, *Response, error) {
	c.initServices()
	return c.Organizations.SearchOrgTeams(ctx, org, opt)
}

// Deprecated: use `client.Organizations.SetPublicOrgMembership` instead.
func (c *Client) SetPublicOrgMembership(ctx context.Context, org, user string, visible bool) (*Response, error) {
	c.initServices()
	return c.Organizations.SetPublicOrgMembership(ctx, org, user, visible)
}

// Deprecated: use `client.Organizations.UnblockOrgUser` instead.
func (c *Client) UnblockOrgUser(ctx context.Context, org, username string) (*Response, error) {
	c.initServices()
	return c.Organizations.UnblockOrgUser(ctx, org, username)
}

// Deprecated: use `client.Organizations.UpdateOrgAvatar` instead.
func (c *Client) UpdateOrgAvatar(ctx context.Context, org string, opt UpdateUserAvatarOption) (*Response, error) {
	c.initServices()
	return c.Organizations.UpdateOrgAvatar(ctx, org, opt)
}

// Deprecated: use `client.Packages.DeletePackage` instead.
func (c *Client) DeletePackage(ctx context.Context, owner, packageType, name, version string) (*Response, error) {
	c.initServices()
	return c.Packages.DeletePackage(ctx, owner, packageType, name, version)
}

// Deprecated: use `client.Packages.GetLatestPackage` instead.
func (c *Client) GetLatestPackage(ctx context.Context, owner, packageType, name string) (*Package, *Response, error) {
	c.initServices()
	return c.Packages.GetLatestPackage(ctx, owner, packageType, name)
}

// Deprecated: use `client.Packages.GetPackage` instead.
func (c *Client) GetPackage(ctx context.Context, owner, packageType, name, version string) (*Package, *Response, error) {
	c.initServices()
	return c.Packages.GetPackage(ctx, owner, packageType, name, version)
}

// Deprecated: use `client.Packages.LinkPackage` instead.
func (c *Client) LinkPackage(ctx context.Context, owner, packageType, name, repoName string) (*Response, error) {
	c.initServices()
	return c.Packages.LinkPackage(ctx, owner, packageType, name, repoName)
}

// Deprecated: use `client.Packages.ListPackageFiles` instead.
func (c *Client) ListPackageFiles(ctx context.Context, owner, packageType, name, version string) ([]*PackageFile, *Response, error) {
	c.initServices()
	return c.Packages.ListPackageFiles(ctx, owner, packageType, name, version)
}

// Deprecated: use `client.Packages.ListPackageVersions` instead.
func (c *Client) ListPackageVersions(ctx context.Context, owner, packageType, name string, opt ListPackagesOptions) ([]*Package, *Response, error) {
	c.initServices()
	return c.Packages.ListPackageVersions(ctx, owner, packageType, name, opt)
}

// Deprecated: use `client.Packages.ListPackages` instead.
func (c *Client) ListPackages(ctx context.Context, owner string, opt ListPackagesOptions) ([]*Package, *Response, error) {
	c.initServices()
	return c.Packages.ListPackages(ctx, owner, opt)
}

// Deprecated: use `client.Packages.UnlinkPackage` instead.
func (c *Client) UnlinkPackage(ctx context.Context, owner, packageType, name string) (*Response, error) {
	c.initServices()
	return c.Packages.UnlinkPackage(ctx, owner, packageType, name)
}

// Deprecated: use `client.PullRequests.CancelScheduledAutoMerge` instead.
func (c *Client) CancelScheduledAutoMerge(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.PullRequests.CancelScheduledAutoMerge(ctx, owner, repo, index)
}

// Deprecated: use `client.PullRequests.CreatePullRequest` instead.
func (c *Client) CreatePullRequest(ctx context.Context, owner, repo string, opt CreatePullRequestOption) (*PullRequest, *Response, error) {
	c.initServices()
	return c.PullRequests.CreatePullRequest(ctx, owner, repo, opt)
}

// Deprecated: use `client.PullRequests.CreatePullReview` instead.
func (c *Client) CreatePullReview(ctx context.Context, owner, repo string, index int64, opt CreatePullReviewOptions) (*PullReview, *Response, error) {
	c.initServices()
	return c.PullRequests.CreatePullReview(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.PullRequests.CreatePullReviewCommentReply` instead.
func (c *Client) CreatePullReviewCommentReply(ctx context.Context, owner, repo string, index, id int64, opt CreatePullReviewCommentReplyOptions) (*PullReviewComment, *Response, error) {
	c.initServices()
	return c.PullRequests.CreatePullReviewCommentReply(ctx, owner, repo, index, id, opt)
}

// Deprecated: use `client.PullRequests.CreateReviewRequests` instead.
func (c *Client) CreateReviewRequests(ctx context.Context, owner, repo string, index int64, opt PullReviewRequestOptions) (*Response, error) {
	c.initServices()
	return c.PullRequests.CreateReviewRequests(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.PullRequests.DeletePullReview` instead.
func (c *Client) DeletePullReview(ctx context.Context, owner, repo string, index, id int64) (*Response, error) {
	c.initServices()
	return c.PullRequests.DeletePullReview(ctx, owner, repo, index, id)
}

// Deprecated: use `client.PullRequests.DeleteReviewRequests` instead.
func (c *Client) DeleteReviewRequests(ctx context.Context, owner, repo string, index int64, opt PullReviewRequestOptions) (*Response, error) {
	c.initServices()
	return c.PullRequests.DeleteReviewRequests(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.PullRequests.DismissPullReview` instead.
func (c *Client) DismissPullReview(ctx context.Context, owner, repo string, index, id int64, opt DismissPullReviewOptions) (*Response, error) {
	c.initServices()
	return c.PullRequests.DismissPullReview(ctx, owner, repo, index, id, opt)
}

// Deprecated: use `client.PullRequests.EditPullRequest` instead.
func (c *Client) EditPullRequest(ctx context.Context, owner, repo string, index int64, opt EditPullRequestOption) (*PullRequest, *Response, error) {
	c.initServices()
	return c.PullRequests.EditPullRequest(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.PullRequests.GetCommitPullRequest` instead.
func (c *Client) GetCommitPullRequest(ctx context.Context, owner, repo, sha string) (*PullRequest, *Response, error) {
	c.initServices()
	return c.PullRequests.GetCommitPullRequest(ctx, owner, repo, sha)
}

// Deprecated: use `client.PullRequests.GetPullRequest` instead.
func (c *Client) GetPullRequest(ctx context.Context, owner, repo string, index int64) (*PullRequest, *Response, error) {
	c.initServices()
	return c.PullRequests.GetPullRequest(ctx, owner, repo, index)
}

// Deprecated: use `client.PullRequests.GetPullRequestByBaseHead` instead.
func (c *Client) GetPullRequestByBaseHead(ctx context.Context, owner, repo, base, head string) (*PullRequest, *Response, error) {
	c.initServices()
	return c.PullRequests.GetPullRequestByBaseHead(ctx, owner, repo, base, head)
}

// Deprecated: use `client.PullRequests.GetPullRequestDiff` instead.
func (c *Client) GetPullRequestDiff(ctx context.Context, owner, repo string, index int64, opts PullRequestDiffOptions) ([]byte, *Response, error) {
	c.initServices()
	return c.PullRequests.GetPullRequestDiff(ctx, owner, repo, index, opts)
}

// Deprecated: use `client.PullRequests.GetPullRequestPatch` instead.
func (c *Client) GetPullRequestPatch(ctx context.Context, owner, repo string, index int64) ([]byte, *Response, error) {
	c.initServices()
	return c.PullRequests.GetPullRequestPatch(ctx, owner, repo, index)
}

// Deprecated: use `client.PullRequests.GetPullReview` instead.
func (c *Client) GetPullReview(ctx context.Context, owner, repo string, index, id int64) (*PullReview, *Response, error) {
	c.initServices()
	return c.PullRequests.GetPullReview(ctx, owner, repo, index, id)
}

// Deprecated: use `client.PullRequests.IsPullRequestMerged` instead.
func (c *Client) IsPullRequestMerged(ctx context.Context, owner, repo string, index int64) (bool, *Response, error) {
	c.initServices()
	return c.PullRequests.IsPullRequestMerged(ctx, owner, repo, index)
}

// Deprecated: use `client.PullRequests.ListPullRequestCommits` instead.
func (c *Client) ListPullRequestCommits(ctx context.Context, owner, repo string, index int64, opt ListPullRequestCommitsOptions) ([]*Commit, *Response, error) {
	c.initServices()
	return c.PullRequests.ListPullRequestCommits(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.PullRequests.ListPullRequestFiles` instead.
func (c *Client) ListPullRequestFiles(ctx context.Context, owner, repo string, index int64, opt ListPullRequestFilesOptions) ([]*ChangedFile, *Response, error) {
	c.initServices()
	return c.PullRequests.ListPullRequestFiles(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.PullRequests.ListPullReviewComments` instead.
func (c *Client) ListPullReviewComments(ctx context.Context, owner, repo string, index, id int64) ([]*PullReviewComment, *Response, error) {
	c.initServices()
	return c.PullRequests.ListPullReviewComments(ctx, owner, repo, index, id)
}

// Deprecated: use `client.PullRequests.ListPullReviews` instead.
func (c *Client) ListPullReviews(ctx context.Context, owner, repo string, index int64, opt ListPullReviewsOptions) ([]*PullReview, *Response, error) {
	c.initServices()
	return c.PullRequests.ListPullReviews(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.PullRequests.ListRepoPinnedPullRequests` instead.
func (c *Client) ListRepoPinnedPullRequests(ctx context.Context, owner, repo string) ([]*PullRequest, *Response, error) {
	c.initServices()
	return c.PullRequests.ListRepoPinnedPullRequests(ctx, owner, repo)
}

// Deprecated: use `client.PullRequests.ListRepoPullRequests` instead.
func (c *Client) ListRepoPullRequests(ctx context.Context, owner, repo string, opt ListPullRequestsOptions) ([]*PullRequest, *Response, error) {
	c.initServices()
	return c.PullRequests.ListRepoPullRequests(ctx, owner, repo, opt)
}

// Deprecated: use `client.PullRequests.MergePullRequest` instead.
func (c *Client) MergePullRequest(ctx context.Context, owner, repo string, index int64, opt MergePullRequestOption) (bool, *Response, error) {
	c.initServices()
	return c.PullRequests.MergePullRequest(ctx, owner, repo, index, opt)
}

// Deprecated: use `client.PullRequests.ResolvePullReviewComment` instead.
func (c *Client) ResolvePullReviewComment(ctx context.Context, owner, repo string, commentID int64) (*Response, error) {
	c.initServices()
	return c.PullRequests.ResolvePullReviewComment(ctx, owner, repo, commentID)
}

// Deprecated: use `client.PullRequests.SubmitPullReview` instead.
func (c *Client) SubmitPullReview(ctx context.Context, owner, repo string, index, id int64, opt SubmitPullReviewOptions) (*PullReview, *Response, error) {
	c.initServices()
	return c.PullRequests.SubmitPullReview(ctx, owner, repo, index, id, opt)
}

// Deprecated: use `client.PullRequests.UnDismissPullReview` instead.
func (c *Client) UnDismissPullReview(ctx context.Context, owner, repo string, index, id int64) (*Response, error) {
	c.initServices()
	return c.PullRequests.UnDismissPullReview(ctx, owner, repo, index, id)
}

// Deprecated: use `client.PullRequests.UnresolvePullReviewComment` instead.
func (c *Client) UnresolvePullReviewComment(ctx context.Context, owner, repo string, commentID int64) (*Response, error) {
	c.initServices()
	return c.PullRequests.UnresolvePullReviewComment(ctx, owner, repo, commentID)
}

// Deprecated: use `client.PullRequests.UpdatePullRequest` instead.
func (c *Client) UpdatePullRequest(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	c.initServices()
	return c.PullRequests.UpdatePullRequest(ctx, owner, repo, index)
}

// Deprecated: use `client.Releases.CreateRelease` instead.
func (c *Client) CreateRelease(ctx context.Context, owner, repo string, opt CreateReleaseOption) (*Release, *Response, error) {
	c.initServices()
	return c.Releases.CreateRelease(ctx, owner, repo, opt)
}

// Deprecated: use `client.Releases.CreateReleaseAttachment` instead.
func (c *Client) CreateReleaseAttachment(ctx context.Context, user, repo string, release int64, file io.Reader, filename string) (*Attachment, *Response, error) {
	c.initServices()
	return c.Releases.CreateReleaseAttachment(ctx, user, repo, release, file, filename)
}

// Deprecated: use `client.Releases.DeleteRelease` instead.
func (c *Client) DeleteRelease(ctx context.Context, user, repo string, id int64) (*Response, error) {
	c.initServices()
	return c.Releases.DeleteRelease(ctx, user, repo, id)
}

// Deprecated: use `client.Releases.DeleteReleaseAttachment` instead.
func (c *Client) DeleteReleaseAttachment(ctx context.Context, user, repo string, release, id int64) (*Response, error) {
	c.initServices()
	return c.Releases.DeleteReleaseAttachment(ctx, user, repo, release, id)
}

// Deprecated: use `client.Releases.DeleteReleaseByTag` instead.
func (c *Client) DeleteReleaseByTag(ctx context.Context, user, repo, tag string) (*Response, error) {
	c.initServices()
	return c.Releases.DeleteReleaseByTag(ctx, user, repo, tag)
}

// Deprecated: use `client.Releases.EditRelease` instead.
func (c *Client) EditRelease(ctx context.Context, owner, repo string, id int64, form EditReleaseOption) (*Release, *Response, error) {
	c.initServices()
	return c.Releases.EditRelease(ctx, owner, repo, id, form)
}

// Deprecated: use `client.Releases.EditReleaseAttachment` instead.
func (c *Client) EditReleaseAttachment(ctx context.Context, user, repo string, release, attachment int64, form EditAttachmentOptions) (*Attachment, *Response, error) {
	c.initServices()
	return c.Releases.EditReleaseAttachment(ctx, user, repo, release, attachment, form)
}

// Deprecated: use `client.Releases.GetLatestRelease` instead.
func (c *Client) GetLatestRelease(ctx context.Context, owner, repo string) (*Release, *Response, error) {
	c.initServices()
	return c.Releases.GetLatestRelease(ctx, owner, repo)
}

// Deprecated: use `client.Releases.GetRelease` instead.
func (c *Client) GetRelease(ctx context.Context, owner, repo string, id int64) (*Release, *Response, error) {
	c.initServices()
	return c.Releases.GetRelease(ctx, owner, repo, id)
}

// Deprecated: use `client.Releases.GetReleaseAttachment` instead.
func (c *Client) GetReleaseAttachment(ctx context.Context, user, repo string, release, id int64) (*Attachment, *Response, error) {
	c.initServices()
	return c.Releases.GetReleaseAttachment(ctx, user, repo, release, id)
}

// Deprecated: use `client.Releases.GetReleaseByTag` instead.
func (c *Client) GetReleaseByTag(ctx context.Context, owner, repo, tag string) (*Release, *Response, error) {
	c.initServices()
	return c.Releases.GetReleaseByTag(ctx, owner, repo, tag)
}

// Deprecated: use `client.Releases.ListReleaseAttachments` instead.
func (c *Client) ListReleaseAttachments(ctx context.Context, user, repo string, release int64, opt ListReleaseAttachmentsOptions) ([]*Attachment, *Response, error) {
	c.initServices()
	return c.Releases.ListReleaseAttachments(ctx, user, repo, release, opt)
}

// Deprecated: use `client.Releases.ListReleases` instead.
func (c *Client) ListReleases(ctx context.Context, owner, repo string, opt ListReleasesOptions) ([]*Release, *Response, error) {
	c.initServices()
	return c.Releases.ListReleases(ctx, owner, repo, opt)
}

// Deprecated: use `client.Render.RenderMarkdown` instead.
func (c *Client) RenderMarkdown(ctx context.Context, opt MarkdownOption) (string, *Response, error) {
	c.initServices()
	return c.Render.RenderMarkdown(ctx, opt)
}

// Deprecated: use `client.Render.RenderMarkdownRaw` instead.
func (c *Client) RenderMarkdownRaw(ctx context.Context, markdown string) (string, *Response, error) {
	c.initServices()
	return c.Render.RenderMarkdownRaw(ctx, markdown)
}

// Deprecated: use `client.Render.RenderMarkup` instead.
func (c *Client) RenderMarkup(ctx context.Context, opt MarkupOption) (string, *Response, error) {
	c.initServices()
	return c.Render.RenderMarkup(ctx, opt)
}

// Deprecated: use `client.Repositories.AcceptRepoTransfer` instead.
func (c *Client) AcceptRepoTransfer(ctx context.Context, owner, reponame string) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.AcceptRepoTransfer(ctx, owner, reponame)
}

// Deprecated: use `client.Repositories.AddCollaborator` instead.
func (c *Client) AddCollaborator(ctx context.Context, user, repo, collaborator string, opt AddCollaboratorOption) (*Response, error) {
	c.initServices()
	return c.Repositories.AddCollaborator(ctx, user, repo, collaborator, opt)
}

// Deprecated: use `client.Repositories.AddRepoTeam` instead.
func (c *Client) AddRepoTeam(ctx context.Context, user, repo, team string) (*Response, error) {
	c.initServices()
	return c.Repositories.AddRepoTeam(ctx, user, repo, team)
}

// Deprecated: use `client.Repositories.AddRepoTopic` instead.
func (c *Client) AddRepoTopic(ctx context.Context, user, repo, topic string) (*Response, error) {
	c.initServices()
	return c.Repositories.AddRepoTopic(ctx, user, repo, topic)
}

// Deprecated: use `client.Repositories.ApplyRepoDiffPatch` instead.
func (c *Client) ApplyRepoDiffPatch(ctx context.Context, owner, repo string, opt ApplyDiffPatchFileOptions) (*FileResponse, *Response, error) {
	c.initServices()
	return c.Repositories.ApplyRepoDiffPatch(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.ChangeFiles` instead.
func (c *Client) ChangeFiles(ctx context.Context, owner, repo string, opt ChangeFilesOptions) (*FileResponse, *Response, error) {
	c.initServices()
	return c.Repositories.ChangeFiles(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.CheckPinAllowed` instead.
func (c *Client) CheckPinAllowed(ctx context.Context, owner, repo string) (*NewIssuePinsAllowed, *Response, error) {
	c.initServices()
	return c.Repositories.CheckPinAllowed(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.CheckRepoTeam` instead.
func (c *Client) CheckRepoTeam(ctx context.Context, user, repo, team string) (*Team, *Response, error) {
	c.initServices()
	return c.Repositories.CheckRepoTeam(ctx, user, repo, team)
}

// Deprecated: use `client.Repositories.CheckRepoWatch` instead.
func (c *Client) CheckRepoWatch(ctx context.Context, owner, repo string) (bool, *Response, error) {
	c.initServices()
	return c.Repositories.CheckRepoWatch(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.CollaboratorPermission` instead.
func (c *Client) CollaboratorPermission(ctx context.Context, user, repo, collaborator string) (*CollaboratorPermissionResult, *Response, error) {
	c.initServices()
	return c.Repositories.CollaboratorPermission(ctx, user, repo, collaborator)
}

// Deprecated: use `client.Repositories.CompareCommits` instead.
func (c *Client) CompareCommits(ctx context.Context, user, repo, prev, current string) (*Compare, *Response, error) {
	c.initServices()
	return c.Repositories.CompareCommits(ctx, user, repo, prev, current)
}

// Deprecated: use `client.Repositories.CreateBranch` instead.
func (c *Client) CreateBranch(ctx context.Context, owner, repo string, opt CreateBranchOption) (*Branch, *Response, error) {
	c.initServices()
	return c.Repositories.CreateBranch(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.CreateBranchProtection` instead.
func (c *Client) CreateBranchProtection(ctx context.Context, owner, repo string, opt CreateBranchProtectionOption) (*BranchProtection, *Response, error) {
	c.initServices()
	return c.Repositories.CreateBranchProtection(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.CreateDeployKey` instead.
func (c *Client) CreateDeployKey(ctx context.Context, user, repo string, opt CreateKeyOption) (*DeployKey, *Response, error) {
	c.initServices()
	return c.Repositories.CreateDeployKey(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.CreateFile` instead.
func (c *Client) CreateFile(ctx context.Context, owner, repo, filepath string, opt CreateFileOptions) (*FileResponse, *Response, error) {
	c.initServices()
	return c.Repositories.CreateFile(ctx, owner, repo, filepath, opt)
}

// Deprecated: use `client.Repositories.CreateFork` instead.
func (c *Client) CreateFork(ctx context.Context, user, repo string, form CreateForkOption) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.CreateFork(ctx, user, repo, form)
}

// Deprecated: use `client.Repositories.CreateLabel` instead.
func (c *Client) CreateLabel(ctx context.Context, owner, repo string, opt CreateLabelOption) (*Label, *Response, error) {
	c.initServices()
	return c.Repositories.CreateLabel(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.CreateMilestone` instead.
func (c *Client) CreateMilestone(ctx context.Context, owner, repo string, opt CreateMilestoneOption) (*Milestone, *Response, error) {
	c.initServices()
	return c.Repositories.CreateMilestone(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.CreateOrgRepo` instead.
func (c *Client) CreateOrgRepo(ctx context.Context, org string, opt CreateRepoOption) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.CreateOrgRepo(ctx, org, opt)
}

// Deprecated: use `client.Repositories.CreateRepo` instead.
func (c *Client) CreateRepo(ctx context.Context, opt CreateRepoOption) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.CreateRepo(ctx, opt)
}

// Deprecated: use `client.Repositories.CreateRepoFromTemplate` instead.
func (c *Client) CreateRepoFromTemplate(ctx context.Context, templateOwner, templateRepo string, opt CreateRepoFromTemplateOption) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.CreateRepoFromTemplate(ctx, templateOwner, templateRepo, opt)
}

// Deprecated: use `client.Repositories.CreateStatus` instead.
func (c *Client) CreateStatus(ctx context.Context, owner, repo, sha string, opts CreateStatusOption) (*Status, *Response, error) {
	c.initServices()
	return c.Repositories.CreateStatus(ctx, owner, repo, sha, opts)
}

// Deprecated: use `client.Repositories.CreateTag` instead.
func (c *Client) CreateTag(ctx context.Context, user, repo string, opt CreateTagOption) (*Tag, *Response, error) {
	c.initServices()
	return c.Repositories.CreateTag(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.CreateTagProtection` instead.
func (c *Client) CreateTagProtection(ctx context.Context, owner, repo string, opt CreateTagProtectionOption) (*TagProtection, *Response, error) {
	c.initServices()
	return c.Repositories.CreateTagProtection(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.DeleteBranchProtection` instead.
func (c *Client) DeleteBranchProtection(ctx context.Context, owner, repo, name string) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteBranchProtection(ctx, owner, repo, name)
}

// Deprecated: use `client.Repositories.DeleteCollaborator` instead.
func (c *Client) DeleteCollaborator(ctx context.Context, user, repo, collaborator string) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteCollaborator(ctx, user, repo, collaborator)
}

// Deprecated: use `client.Repositories.DeleteDeployKey` instead.
func (c *Client) DeleteDeployKey(ctx context.Context, owner, repo string, keyID int64) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteDeployKey(ctx, owner, repo, keyID)
}

// Deprecated: use `client.Repositories.DeleteFile` instead.
func (c *Client) DeleteFile(ctx context.Context, owner, repo, filepath string, opt DeleteFileOptions) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteFile(ctx, owner, repo, filepath, opt)
}

// Deprecated: use `client.Repositories.DeleteLabel` instead.
func (c *Client) DeleteLabel(ctx context.Context, owner, repo string, id int64) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteLabel(ctx, owner, repo, id)
}

// Deprecated: use `client.Repositories.DeleteMilestone` instead.
func (c *Client) DeleteMilestone(ctx context.Context, owner, repo string, id int64) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteMilestone(ctx, owner, repo, id)
}

// Deprecated: use `client.Repositories.DeleteMilestoneByName` instead.
func (c *Client) DeleteMilestoneByName(ctx context.Context, owner, repo, name string) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteMilestoneByName(ctx, owner, repo, name)
}

// Deprecated: use `client.Repositories.DeletePushMirror` instead.
func (c *Client) DeletePushMirror(ctx context.Context, user, repo, remoteName string) (*Response, error) {
	c.initServices()
	return c.Repositories.DeletePushMirror(ctx, user, repo, remoteName)
}

// Deprecated: use `client.Repositories.DeleteRepo` instead.
func (c *Client) DeleteRepo(ctx context.Context, owner, repo string) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteRepo(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.DeleteRepoAvatar` instead.
func (c *Client) DeleteRepoAvatar(ctx context.Context, owner, repo string) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteRepoAvatar(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.DeleteRepoBranch` instead.
func (c *Client) DeleteRepoBranch(ctx context.Context, user, repo, branch string) (bool, *Response, error) {
	c.initServices()
	return c.Repositories.DeleteRepoBranch(ctx, user, repo, branch)
}

// Deprecated: use `client.Repositories.DeleteRepoTopic` instead.
func (c *Client) DeleteRepoTopic(ctx context.Context, user, repo, topic string) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteRepoTopic(ctx, user, repo, topic)
}

// Deprecated: use `client.Repositories.DeleteTag` instead.
func (c *Client) DeleteTag(ctx context.Context, user, repo, tag string) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteTag(ctx, user, repo, tag)
}

// Deprecated: use `client.Repositories.DeleteTagProtection` instead.
func (c *Client) DeleteTagProtection(ctx context.Context, owner, repo string, id int64) (*Response, error) {
	c.initServices()
	return c.Repositories.DeleteTagProtection(ctx, owner, repo, id)
}

// Deprecated: use `client.Repositories.EditBranchProtection` instead.
func (c *Client) EditBranchProtection(ctx context.Context, owner, repo, name string, opt EditBranchProtectionOption) (*BranchProtection, *Response, error) {
	c.initServices()
	return c.Repositories.EditBranchProtection(ctx, owner, repo, name, opt)
}

// Deprecated: use `client.Repositories.EditLabel` instead.
func (c *Client) EditLabel(ctx context.Context, owner, repo string, id int64, opt EditLabelOption) (*Label, *Response, error) {
	c.initServices()
	return c.Repositories.EditLabel(ctx, owner, repo, id, opt)
}

// Deprecated: use `client.Repositories.EditMilestone` instead.
func (c *Client) EditMilestone(ctx context.Context, owner, repo string, id int64, opt EditMilestoneOption) (*Milestone, *Response, error) {
	c.initServices()
	return c.Repositories.EditMilestone(ctx, owner, repo, id, opt)
}

// Deprecated: use `client.Repositories.EditMilestoneByName` instead.
func (c *Client) EditMilestoneByName(ctx context.Context, owner, repo, name string, opt EditMilestoneOption) (*Milestone, *Response, error) {
	c.initServices()
	return c.Repositories.EditMilestoneByName(ctx, owner, repo, name, opt)
}

// Deprecated: use `client.Repositories.EditRepo` instead.
func (c *Client) EditRepo(ctx context.Context, owner, reponame string, opt EditRepoOption) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.EditRepo(ctx, owner, reponame, opt)
}

// Deprecated: use `client.Repositories.EditTagProtection` instead.
func (c *Client) EditTagProtection(ctx context.Context, owner, repo string, id int64, opt EditTagProtectionOption) (*TagProtection, *Response, error) {
	c.initServices()
	return c.Repositories.EditTagProtection(ctx, owner, repo, id, opt)
}

// Deprecated: use `client.Repositories.GetAnnotatedTag` instead.
func (c *Client) GetAnnotatedTag(ctx context.Context, user, repo, sha string) (*AnnotatedTag, *Response, error) {
	c.initServices()
	return c.Repositories.GetAnnotatedTag(ctx, user, repo, sha)
}

// Deprecated: use `client.Repositories.GetArchive` instead.
func (c *Client) GetArchive(ctx context.Context, owner, repo, ref string, ext ArchiveType) ([]byte, *Response, error) {
	c.initServices()
	return c.Repositories.GetArchive(ctx, owner, repo, ref, ext)
}

// Deprecated: use `client.Repositories.GetArchiveReader` instead.
func (c *Client) GetArchiveReader(ctx context.Context, owner, repo, ref string, ext ArchiveType) (io.ReadCloser, *Response, error) {
	c.initServices()
	return c.Repositories.GetArchiveReader(ctx, owner, repo, ref, ext)
}

// Deprecated: use `client.Repositories.GetAssignees` instead.
func (c *Client) GetAssignees(ctx context.Context, user, repo string) ([]*User, *Response, error) {
	c.initServices()
	return c.Repositories.GetAssignees(ctx, user, repo)
}

// Deprecated: use `client.Repositories.GetBranchProtection` instead.
func (c *Client) GetBranchProtection(ctx context.Context, owner, repo, name string) (*BranchProtection, *Response, error) {
	c.initServices()
	return c.Repositories.GetBranchProtection(ctx, owner, repo, name)
}

// Deprecated: use `client.Repositories.GetCombinedStatus` instead.
func (c *Client) GetCombinedStatus(ctx context.Context, owner, repo, ref string) (*CombinedStatus, *Response, error) {
	c.initServices()
	return c.Repositories.GetCombinedStatus(ctx, owner, repo, ref)
}

// Deprecated: use `client.Repositories.GetCommitDiff` instead.
func (c *Client) GetCommitDiff(ctx context.Context, user, repo, commitID string) ([]byte, *Response, error) {
	c.initServices()
	return c.Repositories.GetCommitDiff(ctx, user, repo, commitID)
}

// Deprecated: use `client.Repositories.GetCommitPatch` instead.
func (c *Client) GetCommitPatch(ctx context.Context, user, repo, commitID string) ([]byte, *Response, error) {
	c.initServices()
	return c.Repositories.GetCommitPatch(ctx, user, repo, commitID)
}

// Deprecated: use `client.Repositories.GetContents` instead.
func (c *Client) GetContents(ctx context.Context, owner, repo, ref, filepath string) (*ContentsResponse, *Response, error) {
	c.initServices()
	return c.Repositories.GetContents(ctx, owner, repo, ref, filepath)
}

// Deprecated: use `client.Repositories.GetContentsExt` instead.
func (c *Client) GetContentsExt(ctx context.Context, owner, repo, filepath string, opt GetContentsExtOptions) (*ContentsExtResponse, *Response, error) {
	c.initServices()
	return c.Repositories.GetContentsExt(ctx, owner, repo, filepath, opt)
}

// Deprecated: use `client.Repositories.GetDeployKey` instead.
func (c *Client) GetDeployKey(ctx context.Context, user, repo string, keyID int64) (*DeployKey, *Response, error) {
	c.initServices()
	return c.Repositories.GetDeployKey(ctx, user, repo, keyID)
}

// Deprecated: use `client.Repositories.GetEditorConfig` instead.
func (c *Client) GetEditorConfig(ctx context.Context, owner, repo, filepath string, ref ...string) ([]byte, *Response, error) {
	c.initServices()
	return c.Repositories.GetEditorConfig(ctx, owner, repo, filepath, ref...)
}

// Deprecated: use `client.Repositories.GetFile` instead.
func (c *Client) GetFile(ctx context.Context, owner, repo, ref, filepath string, resolveLFS ...bool) ([]byte, *Response, error) {
	c.initServices()
	return c.Repositories.GetFile(ctx, owner, repo, ref, filepath, resolveLFS...)
}

// Deprecated: use `client.Repositories.GetFileReader` instead.
func (c *Client) GetFileReader(ctx context.Context, owner, repo, ref, filepath string, resolveLFS ...bool) (io.ReadCloser, *Response, error) {
	c.initServices()
	return c.Repositories.GetFileReader(ctx, owner, repo, ref, filepath, resolveLFS...)
}

// Deprecated: use `client.Repositories.GetIssueConfig` instead.
func (c *Client) GetIssueConfig(ctx context.Context, owner, repo string) (*IssueConfig, *Response, error) {
	c.initServices()
	return c.Repositories.GetIssueConfig(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.GetMilestone` instead.
func (c *Client) GetMilestone(ctx context.Context, owner, repo string, id int64) (*Milestone, *Response, error) {
	c.initServices()
	return c.Repositories.GetMilestone(ctx, owner, repo, id)
}

// Deprecated: use `client.Repositories.GetMilestoneByName` instead.
func (c *Client) GetMilestoneByName(ctx context.Context, owner, repo, name string) (*Milestone, *Response, error) {
	c.initServices()
	return c.Repositories.GetMilestoneByName(ctx, owner, repo, name)
}

// Deprecated: use `client.Repositories.GetMyStarredRepos` instead.
func (c *Client) GetMyStarredRepos(ctx context.Context) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.GetMyStarredRepos(ctx)
}

// Deprecated: use `client.Repositories.GetMyWatchedRepos` instead.
func (c *Client) GetMyWatchedRepos(ctx context.Context) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.GetMyWatchedRepos(ctx)
}

// Deprecated: use `client.Repositories.GetPushMirrorByRemoteName` instead.
func (c *Client) GetPushMirrorByRemoteName(ctx context.Context, user, repo, remoteName string) (*PushMirrorResponse, *Response, error) {
	c.initServices()
	return c.Repositories.GetPushMirrorByRemoteName(ctx, user, repo, remoteName)
}

// Deprecated: use `client.Repositories.GetRawFile` instead.
func (c *Client) GetRawFile(ctx context.Context, owner, repo, filepath string, ref ...string) ([]byte, *Response, error) {
	c.initServices()
	return c.Repositories.GetRawFile(ctx, owner, repo, filepath, ref...)
}

// Deprecated: use `client.Repositories.GetRawFileOrLFS` instead.
func (c *Client) GetRawFileOrLFS(ctx context.Context, owner, repo, filepath string, ref ...string) ([]byte, *Response, error) {
	c.initServices()
	return c.Repositories.GetRawFileOrLFS(ctx, owner, repo, filepath, ref...)
}

// Deprecated: use `client.Repositories.GetRepo` instead.
func (c *Client) GetRepo(ctx context.Context, owner, reponame string) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepo(ctx, owner, reponame)
}

// Deprecated: use `client.Repositories.GetRepoBranch` instead.
func (c *Client) GetRepoBranch(ctx context.Context, user, repo, branch string) (*Branch, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepoBranch(ctx, user, repo, branch)
}

// Deprecated: use `client.Repositories.GetRepoByID` instead.
func (c *Client) GetRepoByID(ctx context.Context, id int64) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepoByID(ctx, id)
}

// Deprecated: use `client.Repositories.GetRepoFileContents` instead.
func (c *Client) GetRepoFileContents(ctx context.Context, owner, repo, ref string, opt GetFilesOptions) ([]*ContentsResponse, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepoFileContents(ctx, owner, repo, ref, opt)
}

// Deprecated: use `client.Repositories.GetRepoLabel` instead.
func (c *Client) GetRepoLabel(ctx context.Context, owner, repo string, id int64) (*Label, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepoLabel(ctx, owner, repo, id)
}

// Deprecated: use `client.Repositories.GetRepoLanguages` instead.
func (c *Client) GetRepoLanguages(ctx context.Context, owner, repo string) (map[string]int64, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepoLanguages(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.GetRepoLicenses` instead.
func (c *Client) GetRepoLicenses(ctx context.Context, owner, repo string) ([]string, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepoLicenses(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.GetRepoSigningKeyGPG` instead.
func (c *Client) GetRepoSigningKeyGPG(ctx context.Context, owner, repo string) (string, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepoSigningKeyGPG(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.GetRepoSigningKeySSH` instead.
func (c *Client) GetRepoSigningKeySSH(ctx context.Context, owner, repo string) (string, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepoSigningKeySSH(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.GetRepoTeams` instead.
func (c *Client) GetRepoTeams(ctx context.Context, user, repo string) ([]*Team, *Response, error) {
	c.initServices()
	return c.Repositories.GetRepoTeams(ctx, user, repo)
}

// Deprecated: use `client.Repositories.GetReviewers` instead.
func (c *Client) GetReviewers(ctx context.Context, user, repo string) ([]*User, *Response, error) {
	c.initServices()
	return c.Repositories.GetReviewers(ctx, user, repo)
}

// Deprecated: use `client.Repositories.GetSingleCommit` instead.
func (c *Client) GetSingleCommit(ctx context.Context, user, repo, commitID string) (*Commit, *Response, error) {
	c.initServices()
	return c.Repositories.GetSingleCommit(ctx, user, repo, commitID)
}

// Deprecated: use `client.Repositories.GetStarredRepos` instead.
func (c *Client) GetStarredRepos(ctx context.Context, user string) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.GetStarredRepos(ctx, user)
}

// Deprecated: use `client.Repositories.GetTag` instead.
func (c *Client) GetTag(ctx context.Context, user, repo, tag string) (*Tag, *Response, error) {
	c.initServices()
	return c.Repositories.GetTag(ctx, user, repo, tag)
}

// Deprecated: use `client.Repositories.GetTagProtection` instead.
func (c *Client) GetTagProtection(ctx context.Context, owner, repo string, id int64) (*TagProtection, *Response, error) {
	c.initServices()
	return c.Repositories.GetTagProtection(ctx, owner, repo, id)
}

// Deprecated: use `client.Repositories.GetWatchedRepos` instead.
func (c *Client) GetWatchedRepos(ctx context.Context, user string) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.GetWatchedRepos(ctx, user)
}

// Deprecated: use `client.Repositories.IsCollaborator` instead.
func (c *Client) IsCollaborator(ctx context.Context, user, repo, collaborator string) (bool, *Response, error) {
	c.initServices()
	return c.Repositories.IsCollaborator(ctx, user, repo, collaborator)
}

// Deprecated: use `client.Repositories.IsRepoStarring` instead.
func (c *Client) IsRepoStarring(ctx context.Context, user, repo string) (bool, *Response, error) {
	c.initServices()
	return c.Repositories.IsRepoStarring(ctx, user, repo)
}

// Deprecated: use `client.Repositories.ListBranchProtections` instead.
func (c *Client) ListBranchProtections(ctx context.Context, owner, repo string, opt ListBranchProtectionsOptions) ([]*BranchProtection, *Response, error) {
	c.initServices()
	return c.Repositories.ListBranchProtections(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.ListCollaborators` instead.
func (c *Client) ListCollaborators(ctx context.Context, user, repo string, opt ListCollaboratorsOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Repositories.ListCollaborators(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.ListContents` instead.
func (c *Client) ListContents(ctx context.Context, owner, repo, ref, filepath string) ([]*ContentsResponse, *Response, error) {
	c.initServices()
	return c.Repositories.ListContents(ctx, owner, repo, ref, filepath)
}

// Deprecated: use `client.Repositories.ListDeployKeys` instead.
func (c *Client) ListDeployKeys(ctx context.Context, user, repo string, opt ListDeployKeysOptions) ([]*DeployKey, *Response, error) {
	c.initServices()
	return c.Repositories.ListDeployKeys(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.ListForks` instead.
func (c *Client) ListForks(ctx context.Context, user, repo string, opt ListForksOptions) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.ListForks(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.ListMilestones` instead.
func (c *Client) ListRepoMilestones(ctx context.Context, owner, repo string, opt ListMilestoneOption) ([]*Milestone, *Response, error) {
	c.initServices()
	return c.Repositories.ListMilestones(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.ListMyRepos` instead.
func (c *Client) ListMyRepos(ctx context.Context, opt ListReposOptions) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.ListMyRepos(ctx, opt)
}

// Deprecated: use `client.Repositories.ListOrgRepos` instead.
func (c *Client) ListOrgRepos(ctx context.Context, org string, opt ListOrgReposOptions) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.ListOrgRepos(ctx, org, opt)
}

// Deprecated: use `client.Repositories.ListPushMirrors` instead.
func (c *Client) ListPushMirrors(ctx context.Context, user, repo string, opt ListOptions) ([]*PushMirrorResponse, *Response, error) {
	c.initServices()
	return c.Repositories.ListPushMirrors(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.ListRepoBranches` instead.
func (c *Client) ListRepoBranches(ctx context.Context, user, repo string, opt ListRepoBranchesOptions) ([]*Branch, *Response, error) {
	c.initServices()
	return c.Repositories.ListRepoBranches(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.ListRepoCommits` instead.
func (c *Client) ListRepoCommits(ctx context.Context, user, repo string, opt ListCommitOptions) ([]*Commit, *Response, error) {
	c.initServices()
	return c.Repositories.ListRepoCommits(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.ListRepoLabels` instead.
func (c *Client) ListRepoLabels(ctx context.Context, owner, repo string, opt ListLabelsOptions) ([]*Label, *Response, error) {
	c.initServices()
	return c.Repositories.ListRepoLabels(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.ListRepoStargazers` instead.
func (c *Client) ListRepoStargazers(ctx context.Context, user, repo string, opt ListStargazersOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Repositories.ListRepoStargazers(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.ListRepoSubscribers` instead.
func (c *Client) ListRepoSubscribers(ctx context.Context, owner, repo string, opt ListOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Repositories.ListRepoSubscribers(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.ListRepoTags` instead.
func (c *Client) ListRepoTags(ctx context.Context, user, repo string, opt ListRepoTagsOptions) ([]*Tag, *Response, error) {
	c.initServices()
	return c.Repositories.ListRepoTags(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.ListRepoTopics` instead.
func (c *Client) ListRepoTopics(ctx context.Context, user, repo string, opt ListRepoTopicsOptions) ([]string, *Response, error) {
	c.initServices()
	return c.Repositories.ListRepoTopics(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.ListStatuses` instead.
func (c *Client) ListStatuses(ctx context.Context, owner, repo, ref string, opt ListStatusesOption) ([]*Status, *Response, error) {
	c.initServices()
	return c.Repositories.ListStatuses(ctx, owner, repo, ref, opt)
}

// Deprecated: use `client.Repositories.ListTagProtection` instead.
func (c *Client) ListTagProtection(ctx context.Context, owner, repo string, opt ListRepoTagProtectionsOptions) ([]*TagProtection, *Response, error) {
	c.initServices()
	return c.Repositories.ListTagProtection(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.ListUserRepos` instead.
func (c *Client) ListUserRepos(ctx context.Context, user string, opt ListReposOptions) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.ListUserRepos(ctx, user, opt)
}

// Deprecated: use `client.Repositories.MergeUpstream` instead.
func (c *Client) MergeUpstream(ctx context.Context, owner, repo string, opt MergeUpstreamRequest) (*MergeUpstreamResponse, *Response, error) {
	c.initServices()
	return c.Repositories.MergeUpstream(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.MigrateRepo` instead.
func (c *Client) MigrateRepo(ctx context.Context, opt MigrateRepoOption) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.MigrateRepo(ctx, opt)
}

// Deprecated: use `client.Repositories.MirrorSync` instead.
func (c *Client) MirrorSync(ctx context.Context, owner, repo string) (*Response, error) {
	c.initServices()
	return c.Repositories.MirrorSync(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.PostRepoFileContents` instead.
func (c *Client) PostRepoFileContents(ctx context.Context, owner, repo, ref string, opt GetFilesOptions) ([]*ContentsResponse, *Response, error) {
	c.initServices()
	return c.Repositories.PostRepoFileContents(ctx, owner, repo, ref, opt)
}

// Deprecated: use `client.Repositories.PushMirrors` instead.
func (c *Client) PushMirrors(ctx context.Context, user, repo string, opt CreatePushMirrorOption) (*PushMirrorResponse, *Response, error) {
	c.initServices()
	return c.Repositories.PushMirrors(ctx, user, repo, opt)
}

// Deprecated: use `client.Repositories.RejectRepoTransfer` instead.
func (c *Client) RejectRepoTransfer(ctx context.Context, owner, reponame string) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.RejectRepoTransfer(ctx, owner, reponame)
}

// Deprecated: use `client.Repositories.RemoveRepoTeam` instead.
func (c *Client) RemoveRepoTeam(ctx context.Context, user, repo, team string) (*Response, error) {
	c.initServices()
	return c.Repositories.RemoveRepoTeam(ctx, user, repo, team)
}

// Deprecated: use `client.Repositories.RenameRepoBranch` instead.
func (c *Client) RenameRepoBranch(ctx context.Context, user, repo, branch string, opt RenameRepoBranchOption) (successful bool, resp *Response, err error) {
	c.initServices()
	return c.Repositories.RenameRepoBranch(ctx, user, repo, branch, opt)
}

// Deprecated: use `client.Repositories.SearchRepos` instead.
func (c *Client) SearchRepos(ctx context.Context, opt SearchRepoOptions) ([]*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.SearchRepos(ctx, opt)
}

// Deprecated: use `client.Repositories.SearchTopics` instead.
func (c *Client) SearchTopics(ctx context.Context, opt TopicSearchOptions) (*TopicSearchResult, *Response, error) {
	c.initServices()
	return c.Repositories.SearchTopics(ctx, opt)
}

// Deprecated: use `client.Repositories.SetRepoTopics` instead.
func (c *Client) SetRepoTopics(ctx context.Context, user, repo string, list []string) (*Response, error) {
	c.initServices()
	return c.Repositories.SetRepoTopics(ctx, user, repo, list)
}

// Deprecated: use `client.Repositories.StarRepo` instead.
func (c *Client) StarRepo(ctx context.Context, user, repo string) (*Response, error) {
	c.initServices()
	return c.Repositories.StarRepo(ctx, user, repo)
}

// Deprecated: use `client.Repositories.TransferRepo` instead.
func (c *Client) TransferRepo(ctx context.Context, owner, reponame string, opt TransferRepoOption) (*Repository, *Response, error) {
	c.initServices()
	return c.Repositories.TransferRepo(ctx, owner, reponame, opt)
}

// Deprecated: use `client.Repositories.TriggerPushMirrorsSync` instead.
func (c *Client) TriggerPushMirrorsSync(ctx context.Context, owner, repo string) (*Response, error) {
	c.initServices()
	return c.Repositories.TriggerPushMirrorsSync(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.UnStarRepo` instead.
func (c *Client) UnStarRepo(ctx context.Context, user, repo string) (*Response, error) {
	c.initServices()
	return c.Repositories.UnStarRepo(ctx, user, repo)
}

// Deprecated: use `client.Repositories.UnWatchRepo` instead.
func (c *Client) UnWatchRepo(ctx context.Context, owner, repo string) (*Response, error) {
	c.initServices()
	return c.Repositories.UnWatchRepo(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.UpdateBranchProtectionPriorities` instead.
func (c *Client) UpdateBranchProtectionPriorities(ctx context.Context, owner, repo string, ids []int64) (*Response, error) {
	c.initServices()
	return c.Repositories.UpdateBranchProtectionPriorities(ctx, owner, repo, ids)
}

// Deprecated: use `client.Repositories.UpdateFile` instead.
func (c *Client) UpdateFile(ctx context.Context, owner, repo, filepath string, opt UpdateFileOptions) (*FileResponse, *Response, error) {
	c.initServices()
	return c.Repositories.UpdateFile(ctx, owner, repo, filepath, opt)
}

// Deprecated: use `client.Repositories.UpdateRepoAvatar` instead.
func (c *Client) UpdateRepoAvatar(ctx context.Context, owner, repo string, opt UpdateRepoAvatarOption) (*Response, error) {
	c.initServices()
	return c.Repositories.UpdateRepoAvatar(ctx, owner, repo, opt)
}

// Deprecated: use `client.Repositories.UpdateRepoBranchRef` instead.
func (c *Client) UpdateRepoBranchRef(ctx context.Context, user, repo, branch string, opt UpdateRepoBranchRefOption) (successful bool, resp *Response, err error) {
	c.initServices()
	return c.Repositories.UpdateRepoBranchRef(ctx, user, repo, branch, opt)
}

// Deprecated: use `client.Repositories.ValidateIssueConfig` instead.
func (c *Client) ValidateIssueConfig(ctx context.Context, owner, repo string) (*IssueConfigValidation, *Response, error) {
	c.initServices()
	return c.Repositories.ValidateIssueConfig(ctx, owner, repo)
}

// Deprecated: use `client.Repositories.WatchRepo` instead.
func (c *Client) WatchRepo(ctx context.Context, owner, repo string) (*Response, error) {
	c.initServices()
	return c.Repositories.WatchRepo(ctx, owner, repo)
}

// Deprecated: use `client.Settings.GetGlobalAPISettings` instead.
func (c *Client) GetGlobalAPISettings(ctx context.Context) (*GlobalAPISettings, *Response, error) {
	c.initServices()
	return c.Settings.GetGlobalAPISettings(ctx)
}

// Deprecated: use `client.Settings.GetGlobalAttachmentSettings` instead.
func (c *Client) GetGlobalAttachmentSettings(ctx context.Context) (*GlobalAttachmentSettings, *Response, error) {
	c.initServices()
	return c.Settings.GetGlobalAttachmentSettings(ctx)
}

// Deprecated: use `client.Settings.GetGlobalRepoSettings` instead.
func (c *Client) GetGlobalRepoSettings(ctx context.Context) (*GlobalRepoSettings, *Response, error) {
	c.initServices()
	return c.Settings.GetGlobalRepoSettings(ctx)
}

// Deprecated: use `client.Settings.GetGlobalUISettings` instead.
func (c *Client) GetGlobalUISettings(ctx context.Context) (*GlobalUISettings, *Response, error) {
	c.initServices()
	return c.Settings.GetGlobalUISettings(ctx)
}

// Deprecated: use `client.Templates.GetGitignore` instead.
func (c *Client) GetGitignoreTemplateInfo(ctx context.Context, name string) (*GitignoreTemplateInfo, *Response, error) {
	c.initServices()
	return c.Templates.GetGitignore(ctx, name)
}

// Deprecated: use `client.Templates.GetLabel` instead.
func (c *Client) GetLabelTemplate(ctx context.Context, name string) ([]*LabelTemplate, *Response, error) {
	c.initServices()
	return c.Templates.GetLabel(ctx, name)
}

// Deprecated: use `client.Templates.GetLicense` instead.
func (c *Client) GetLicenseTemplateInfo(ctx context.Context, name string) (*LicenseTemplateInfo, *Response, error) {
	c.initServices()
	return c.Templates.GetLicense(ctx, name)
}

// Deprecated: use `client.Templates.ListGitignores` instead.
func (c *Client) ListGitignoresTemplates(ctx context.Context) ([]string, *Response, error) {
	c.initServices()
	return c.Templates.ListGitignores(ctx)
}

// Deprecated: use `client.Templates.ListLabels` instead.
func (c *Client) ListLabelTemplates(ctx context.Context) ([]string, *Response, error) {
	c.initServices()
	return c.Templates.ListLabels(ctx)
}

// Deprecated: use `client.Templates.ListLicenses` instead.
func (c *Client) ListLicenseTemplates(ctx context.Context) ([]*LicensesTemplateListEntry, *Response, error) {
	c.initServices()
	return c.Templates.ListLicenses(ctx)
}

// Deprecated: use `client.Users.AddEmail` instead.
func (c *Client) AddEmail(ctx context.Context, opt CreateEmailOption) ([]*Email, *Response, error) {
	c.initServices()
	return c.Users.AddEmail(ctx, opt)
}

// Deprecated: use `client.Users.BlockUser` instead.
func (c *Client) BlockUser(ctx context.Context, username string) (*Response, error) {
	c.initServices()
	return c.Users.BlockUser(ctx, username)
}

// Deprecated: use `client.Users.CheckUserBlock` instead.
func (c *Client) CheckUserBlock(ctx context.Context, username string) (bool, *Response, error) {
	c.initServices()
	return c.Users.CheckUserBlock(ctx, username)
}

// Deprecated: use `client.Users.CreateAccessToken` instead.
func (c *Client) CreateAccessToken(ctx context.Context, opt CreateAccessTokenOption) (*AccessToken, *Response, error) {
	c.initServices()
	return c.Users.CreateAccessToken(ctx, opt)
}

// Deprecated: use `client.Users.CreateGPGKey` instead.
func (c *Client) CreateGPGKey(ctx context.Context, opt CreateGPGKeyOption) (*GPGKey, *Response, error) {
	c.initServices()
	return c.Users.CreateGPGKey(ctx, opt)
}

// Deprecated: use `client.Users.CreatePublicKey` instead.
func (c *Client) CreatePublicKey(ctx context.Context, opt CreateKeyOption) (*PublicKey, *Response, error) {
	c.initServices()
	return c.Users.CreatePublicKey(ctx, opt)
}

// Deprecated: use `client.Users.DeleteAccessToken` instead.
func (c *Client) DeleteAccessToken(ctx context.Context, value interface{}) (*Response, error) {
	c.initServices()
	return c.Users.DeleteAccessToken(ctx, value)
}

// Deprecated: use `client.Users.DeleteEmail` instead.
func (c *Client) DeleteEmail(ctx context.Context, opt DeleteEmailOption) (*Response, error) {
	c.initServices()
	return c.Users.DeleteEmail(ctx, opt)
}

// Deprecated: use `client.Users.DeleteGPGKey` instead.
func (c *Client) DeleteGPGKey(ctx context.Context, keyID int64) (*Response, error) {
	c.initServices()
	return c.Users.DeleteGPGKey(ctx, keyID)
}

// Deprecated: use `client.Users.DeletePublicKey` instead.
func (c *Client) DeletePublicKey(ctx context.Context, keyID int64) (*Response, error) {
	c.initServices()
	return c.Users.DeletePublicKey(ctx, keyID)
}

// Deprecated: use `client.Users.DeleteUserAvatar` instead.
func (c *Client) DeleteUserAvatar(ctx context.Context) (*Response, error) {
	c.initServices()
	return c.Users.DeleteUserAvatar(ctx)
}

// Deprecated: use `client.Users.Follow` instead.
func (c *Client) Follow(ctx context.Context, target string) (*Response, error) {
	c.initServices()
	return c.Users.Follow(ctx, target)
}

// Deprecated: use `client.Users.GetGPGKey` instead.
func (c *Client) GetGPGKey(ctx context.Context, keyID int64) (*GPGKey, *Response, error) {
	c.initServices()
	return c.Users.GetGPGKey(ctx, keyID)
}

// Deprecated: use `client.Users.GetGPGKeyVerificationToken` instead.
func (c *Client) GetGPGKeyVerificationToken(ctx context.Context) (string, *Response, error) {
	c.initServices()
	return c.Users.GetGPGKeyVerificationToken(ctx)
}

// Deprecated: use `client.Users.GetMyUserInfo` instead.
func (c *Client) GetMyUserInfo(ctx context.Context) (*User, *Response, error) {
	c.initServices()
	return c.Users.GetMyUserInfo(ctx)
}

// Deprecated: use `client.Users.GetPublicKey` instead.
func (c *Client) GetPublicKey(ctx context.Context, keyID int64) (*PublicKey, *Response, error) {
	c.initServices()
	return c.Users.GetPublicKey(ctx, keyID)
}

// Deprecated: use `client.Users.GetUserByID` instead.
func (c *Client) GetUserByID(ctx context.Context, id int64) (*User, *Response, error) {
	c.initServices()
	return c.Users.GetUserByID(ctx, id)
}

// Deprecated: use `client.Users.GetUserInfo` instead.
func (c *Client) GetUserInfo(ctx context.Context, user string) (*User, *Response, error) {
	c.initServices()
	return c.Users.GetUserInfo(ctx, user)
}

// Deprecated: use `client.Users.GetUserSettings` instead.
func (c *Client) GetUserSettings(ctx context.Context) (*UserSettings, *Response, error) {
	c.initServices()
	return c.Users.GetUserSettings(ctx)
}

// Deprecated: use `client.Users.IsFollowing` instead.
func (c *Client) IsFollowing(ctx context.Context, target string) (bool, *Response) {
	c.initServices()
	return c.Users.IsFollowing(ctx, target)
}

// Deprecated: use `client.Users.IsUserFollowing` instead.
func (c *Client) IsUserFollowing(ctx context.Context, user, target string) (bool, *Response) {
	c.initServices()
	return c.Users.IsUserFollowing(ctx, user, target)
}

// Deprecated: use `client.Users.ListAccessTokens` instead.
func (c *Client) ListAccessTokens(ctx context.Context, opts ListAccessTokensOptions) ([]*AccessToken, *Response, error) {
	c.initServices()
	return c.Users.ListAccessTokens(ctx, opts)
}

// Deprecated: use `client.Users.ListEmails` instead.
func (c *Client) ListEmails(ctx context.Context, opt ListEmailsOptions) ([]*Email, *Response, error) {
	c.initServices()
	return c.Users.ListEmails(ctx, opt)
}

// Deprecated: use `client.Users.ListFollowers` instead.
func (c *Client) ListFollowers(ctx context.Context, user string, opt ListFollowersOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Users.ListFollowers(ctx, user, opt)
}

// Deprecated: use `client.Users.ListFollowing` instead.
func (c *Client) ListFollowing(ctx context.Context, user string, opt ListFollowingOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Users.ListFollowing(ctx, user, opt)
}

// Deprecated: use `client.Users.ListGPGKeys` instead.
func (c *Client) ListGPGKeys(ctx context.Context, user string, opt ListGPGKeysOptions) ([]*GPGKey, *Response, error) {
	c.initServices()
	return c.Users.ListGPGKeys(ctx, user, opt)
}

// Deprecated: use `client.Users.ListMyBlocks` instead.
func (c *Client) ListMyBlocks(ctx context.Context, opt ListUserBlocksOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Users.ListMyBlocks(ctx, opt)
}

// Deprecated: use `client.Users.ListMyFollowers` instead.
func (c *Client) ListMyFollowers(ctx context.Context, opt ListFollowersOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Users.ListMyFollowers(ctx, opt)
}

// Deprecated: use `client.Users.ListMyFollowing` instead.
func (c *Client) ListMyFollowing(ctx context.Context, opt ListFollowingOptions) ([]*User, *Response, error) {
	c.initServices()
	return c.Users.ListMyFollowing(ctx, opt)
}

// Deprecated: use `client.Users.ListMyGPGKeys` instead.
func (c *Client) ListMyGPGKeys(ctx context.Context, opt *ListGPGKeysOptions) ([]*GPGKey, *Response, error) {
	c.initServices()
	return c.Users.ListMyGPGKeys(ctx, opt)
}

// Deprecated: use `client.Users.ListMyPublicKeys` instead.
func (c *Client) ListMyPublicKeys(ctx context.Context, opt ListPublicKeysOptions) ([]*PublicKey, *Response, error) {
	c.initServices()
	return c.Users.ListMyPublicKeys(ctx, opt)
}

// Deprecated: use `client.Users.ListPublicKeys` instead.
func (c *Client) ListPublicKeys(ctx context.Context, user string, opt ListPublicKeysOptions) ([]*PublicKey, *Response, error) {
	c.initServices()
	return c.Users.ListPublicKeys(ctx, user, opt)
}

// Deprecated: use `client.Users.SearchUsers` instead.
func (c *Client) SearchUsers(ctx context.Context, opt SearchUsersOption) ([]*User, *Response, error) {
	c.initServices()
	return c.Users.SearchUsers(ctx, opt)
}

// Deprecated: use `client.Users.UnblockUser` instead.
func (c *Client) UnblockUser(ctx context.Context, username string) (*Response, error) {
	c.initServices()
	return c.Users.UnblockUser(ctx, username)
}

// Deprecated: use `client.Users.Unfollow` instead.
func (c *Client) Unfollow(ctx context.Context, target string) (*Response, error) {
	c.initServices()
	return c.Users.Unfollow(ctx, target)
}

// Deprecated: use `client.Users.UpdateUserAvatar` instead.
func (c *Client) UpdateUserAvatar(ctx context.Context, opt UpdateUserAvatarOption) (*Response, error) {
	c.initServices()
	return c.Users.UpdateUserAvatar(ctx, opt)
}

// Deprecated: use `client.Users.UpdateUserSettings` instead.
func (c *Client) UpdateUserSettings(ctx context.Context, opt UserSettingsOptions) (*UserSettings, *Response, error) {
	c.initServices()
	return c.Users.UpdateUserSettings(ctx, opt)
}

// Deprecated: use `client.Users.VerifyGPGKey` instead.
func (c *Client) VerifyGPGKey(ctx context.Context, opt VerifyGPGKeyOption) (*GPGKey, *Response, error) {
	c.initServices()
	return c.Users.VerifyGPGKey(ctx, opt)
}

// Deprecated: use `client.Wiki.CreatePage` instead.
func (c *Client) CreateWikiPage(ctx context.Context, owner, repo string, opt CreateWikiPageOptions) (*WikiPage, *Response, error) {
	c.initServices()
	return c.Wiki.CreatePage(ctx, owner, repo, opt)
}

// Deprecated: use `client.Wiki.DeletePage` instead.
func (c *Client) DeleteWikiPage(ctx context.Context, owner, repo, pageName string) (*Response, error) {
	c.initServices()
	return c.Wiki.DeletePage(ctx, owner, repo, pageName)
}

// Deprecated: use `client.Wiki.EditPage` instead.
func (c *Client) EditWikiPage(ctx context.Context, owner, repo, pageName string, opt CreateWikiPageOptions) (*WikiPage, *Response, error) {
	c.initServices()
	return c.Wiki.EditPage(ctx, owner, repo, pageName, opt)
}

// Deprecated: use `client.Wiki.GetPage` instead.
func (c *Client) GetWikiPage(ctx context.Context, owner, repo, pageName string) (*WikiPage, *Response, error) {
	c.initServices()
	return c.Wiki.GetPage(ctx, owner, repo, pageName)
}

// Deprecated: use `client.Wiki.GetPageRevisions` instead.
func (c *Client) GetWikiPageRevisions(ctx context.Context, owner, repo, pageName string, opt ListWikiPageRevisionsOptions) (*WikiCommitList, *Response, error) {
	c.initServices()
	return c.Wiki.GetPageRevisions(ctx, owner, repo, pageName, opt)
}

// Deprecated: use `client.Wiki.ListPages` instead.
func (c *Client) ListWikiPages(ctx context.Context, owner, repo string, opt ListWikiPagesOptions) ([]*WikiPageMetaData, *Response, error) {
	c.initServices()
	return c.Wiki.ListPages(ctx, owner, repo, opt)
}
