// Copyright 2020 The Gitea Authors. All rights reserved.
// SPDX-License-Identifier: MIT

package cmd

var (
	catSetup    = "SETUP"
	catEntities = "ENTITIES"
	catHelpers  = "HELPERS"
	catMisc     = "MISCELLANEOUS"
)
