// Copyright 2020 The Gitea Authors. All rights reserved.
// SPDX-License-Identifier: MIT

package releases

import (
	stdctx "context"
	"fmt"
	"net/http"
	"os"
	"path/filepath"

	"gitea.dev/sdk"

	"gitea.dev/tea/cmd/flags"
	"gitea.dev/tea/modules/context"
	"github.com/urfave/cli/v3"
)

// CmdReleaseCreate represents a sub command of Release to create release
var CmdReleaseCreate = cli.Command{
	Name:        "create",
	Aliases:     []string{"c"},
	Usage:       "Create a release",
	Description: `Create a release for a new or existing git tag`,
	ArgsUsage:   "[<tag>]",
	Action:      runReleaseCreate,
	Flags: append([]cli.Flag{
		&cli.StringFlag{
			Name:  "tag",
			Usage: "Tag name. If the tag does not exist yet, it will be created by Gitea",
		},
		&cli.StringFlag{
			Name:  "target",
			Usage: "Target branch name or commit hash. Defaults to the default branch of the repo",
		},
		&cli.StringFlag{
			Name:    "title",
			Aliases: []string{"t"},
			Usage:   "Release title",
		},
		&cli.StringFlag{
			Name:    "note",
			Aliases: []string{"n"},
			Usage:   "Release notes",
		},
		&cli.StringFlag{
			Name:    "note-file",
			Aliases: []string{"f"},
			Usage:   "Release notes file name. If set, --note is ignored.",
		},
		&cli.BoolFlag{
			Name:    "draft",
			Aliases: []string{"d"},
			Usage:   "Is a draft",
		},
		&cli.BoolFlag{
			Name:    "prerelease",
			Aliases: []string{"p"},
			Usage:   "Is a pre-release",
		},
		&cli.StringSliceFlag{
			Name:    "asset",
			Aliases: []string{"a"},
			Usage:   "Path to file attachment. Can be specified multiple times",
		},
	}, flags.AllDefaultFlags...),
}

func runReleaseCreate(requestCtx stdctx.Context, cmd *cli.Command) error {
	ctx, err := context.InitCommand(cmd)
	if err != nil {
		return err
	}
	if err := ctx.Ensure(context.CtxRequirement{RemoteRepo: true}); err != nil {
		return err
	}

	tag := ctx.String("tag")
	if cmd.Args().Present() {
		if len(tag) != 0 {
			return fmt.Errorf("ambiguous arguments: provide tagname via --tag or argument, but not both")
		}
		tag = cmd.Args().First()
	}

	notestring := ctx.String("note")
	notefile := ctx.String("note-file")
	if notefile != "" {
		notebytes, err := os.ReadFile(notefile)
		if err != nil {
			return fmt.Errorf("unable to read the note file")
		}
		notestring = string(notebytes)
	}

	release, resp, err := ctx.Login.Client().Releases.CreateRelease(requestCtx, ctx.Owner, ctx.Repo, gitea.CreateReleaseOption{
		TagName:      tag,
		Target:       ctx.String("target"),
		Title:        ctx.String("title"),
		Note:         notestring,
		IsDraft:      ctx.Bool("draft"),
		IsPrerelease: ctx.Bool("prerelease"),
	})
	if err != nil {
		if resp != nil && resp.StatusCode == http.StatusConflict {
			return fmt.Errorf("there is already a release for this tag")
		}
		return err
	}

	for _, asset := range ctx.StringSlice("asset") {
		var file *os.File

		if file, err = os.Open(asset); err != nil {
			return err
		}

		filePath := filepath.Base(asset)

		if _, _, err = ctx.Login.Client().Releases.CreateReleaseAttachment(requestCtx, ctx.Owner, ctx.Repo, release.ID, file, filePath); err != nil {
			file.Close()
			return err
		}

		file.Close()
	}

	return nil
}
