// Copyright 2017 The Gitea Authors. All rights reserved.
// Use of this source code is governed by a MIT-style
// license that can be found in the LICENSE file.

package gitea

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"net/url"
	"time"
)

// TrackedTime worked time for an issue / pr
type TrackedTime struct {
	ID      int64     `json:"id"`
	Created time.Time `json:"created"`
	// Time in seconds
	Time int64 `json:"time"`
	// deprecated (only for backwards compatibility)
	UserID   int64  `json:"user_id"`
	UserName string `json:"user_name"`
	// deprecated (only for backwards compatibility)
	IssueID int64  `json:"issue_id"`
	Issue   *Issue `json:"issue"`
}

// ListTrackedTimesOptions options for listing repository's tracked times
type ListTrackedTimesOptions struct {
	ListOptions
	Since  time.Time
	Before time.Time
	// User filter is only used by ListRepoTrackedTimes !!!
	User string
}

// QueryEncode turns options into querystring argument
func (opt *ListTrackedTimesOptions) QueryEncode() string {
	query := opt.getURLQuery()

	if !opt.Since.IsZero() {
		query.Add("since", opt.Since.Format(time.RFC3339))
	}
	if !opt.Before.IsZero() {
		query.Add("before", opt.Before.Format(time.RFC3339))
	}

	if len(opt.User) != 0 {
		query.Add("user", opt.User)
	}

	return query.Encode()
}

// ListRepoTrackedTimes list tracked times of a repository
func (c *IssuesService) ListRepoTrackedTimes(ctx context.Context, owner, repo string, opt ListTrackedTimesOptions) ([]*TrackedTime, *Response, error) {
	if err := escapeValidatePathSegments(&owner, &repo); err != nil {
		return nil, nil, err
	}
	link, _ := url.Parse(fmt.Sprintf("/repos/%s/%s/times", owner, repo))
	opt.setDefaults()
	link.RawQuery = opt.QueryEncode()
	times := make([]*TrackedTime, 0, opt.PageSize)
	resp, err := c.getParsedResponse(ctx, "GET", link.String(), jsonHeader, nil, &times)
	return times, resp, err
}

// ListMyTrackedTimes list tracked times of the current user with pagination and filtering
func (c *IssuesService) ListMyTrackedTimes(ctx context.Context, opt ListTrackedTimesOptions) ([]*TrackedTime, *Response, error) {
	link, _ := url.Parse("/user/times")
	opt.setDefaults()
	link.RawQuery = opt.QueryEncode()
	times := make([]*TrackedTime, 0, opt.PageSize)
	resp, err := c.getParsedResponse(ctx, "GET", link.String(), jsonHeader, nil, &times)
	return times, resp, err
}

// GetUserTrackedTimes gets all tracked times for a user in a repository.
func (c *IssuesService) GetUserTrackedTimes(ctx context.Context, owner, repo, user string) ([]*TrackedTime, *Response, error) {
	if err := escapeValidatePathSegments(&owner, &repo, &user); err != nil {
		return nil, nil, err
	}
	times := make([]*TrackedTime, 0, 10)
	resp, err := c.getParsedResponse(ctx, "GET",
		fmt.Sprintf("/repos/%s/%s/times/%s", owner, repo, user),
		jsonHeader, nil, &times)
	return times, resp, err
}

// AddTimeOption options for adding time to an issue
type AddTimeOption struct {
	// time in seconds
	Time int64 `json:"time"`
	// optional
	Created time.Time `json:"created"`
	// optional
	User string `json:"user_name"`
}

// Validate the AddTimeOption struct
func (opt AddTimeOption) Validate() error {
	if opt.Time == 0 {
		return fmt.Errorf("no time to add")
	}
	return nil
}

// AddTime adds time to issue with the given index
func (c *IssuesService) AddTime(ctx context.Context, owner, repo string, index int64, opt AddTimeOption) (*TrackedTime, *Response, error) {
	if err := escapeValidatePathSegments(&owner, &repo); err != nil {
		return nil, nil, err
	}
	if err := opt.Validate(); err != nil {
		return nil, nil, err
	}
	body, err := json.Marshal(&opt)
	if err != nil {
		return nil, nil, err
	}
	t := new(TrackedTime)
	resp, err := c.getParsedResponse(ctx, "POST",
		fmt.Sprintf("/repos/%s/%s/issues/%d/times", owner, repo, index),
		jsonHeader, bytes.NewReader(body), t)
	return t, resp, err
}

// ListIssueTrackedTimes list tracked times of a single issue for a given repository
func (c *IssuesService) ListIssueTrackedTimes(ctx context.Context, owner, repo string, index int64, opt ListTrackedTimesOptions) ([]*TrackedTime, *Response, error) {
	if err := escapeValidatePathSegments(&owner, &repo); err != nil {
		return nil, nil, err
	}
	link, _ := url.Parse(fmt.Sprintf("/repos/%s/%s/issues/%d/times", owner, repo, index))
	opt.setDefaults()
	link.RawQuery = opt.QueryEncode()
	times := make([]*TrackedTime, 0, opt.PageSize)
	resp, err := c.getParsedResponse(ctx, "GET", link.String(), jsonHeader, nil, &times)
	return times, resp, err
}

// ResetIssueTime reset tracked time of a single issue for a given repository
func (c *IssuesService) ResetIssueTime(ctx context.Context, owner, repo string, index int64) (*Response, error) {
	if err := escapeValidatePathSegments(&owner, &repo); err != nil {
		return nil, err
	}
	return c.doRequestWithStatusHandle(ctx, "DELETE", fmt.Sprintf("/repos/%s/%s/issues/%d/times", owner, repo, index), jsonHeader, nil)
}

// DeleteTime delete a specific tracked time by id of a single issue for a given repository
func (c *IssuesService) DeleteTime(ctx context.Context, owner, repo string, index, timeID int64) (*Response, error) {
	if err := escapeValidatePathSegments(&owner, &repo); err != nil {
		return nil, err
	}
	return c.doRequestWithStatusHandle(ctx, "DELETE", fmt.Sprintf("/repos/%s/%s/issues/%d/times/%d", owner, repo, index, timeID), jsonHeader, nil)
}
