// Copyright 2020 The Gitea Authors. All rights reserved.
// SPDX-License-Identifier: MIT

package releases

import (
	stdctx "context"
	"fmt"

	"gitea.dev/tea/cmd/flags"
	"gitea.dev/tea/modules/context"

	"github.com/urfave/cli/v3"
)

// CmdReleaseDelete represents a sub command of Release to delete a release
var CmdReleaseDelete = cli.Command{
	Name:        "delete",
	Aliases:     []string{"rm"},
	Usage:       "Delete one or more releases",
	Description: `Delete one or more releases`,
	ArgsUsage:   "<release tag> [<release tag>...]",
	Action:      runReleaseDelete,
	Flags: append([]cli.Flag{
		&cli.BoolFlag{
			Name:    "confirm",
			Aliases: []string{"y"},
			Usage:   "Confirm deletion (required)",
		},
		&cli.BoolFlag{
			Name:  "delete-tag",
			Usage: "Also delete the git tag for this release",
		},
	}, flags.AllDefaultFlags...),
}

func runReleaseDelete(requestCtx stdctx.Context, cmd *cli.Command) error {
	ctx, err := context.InitCommand(cmd)
	if err != nil {
		return err
	}
	if err := ctx.Ensure(context.CtxRequirement{RemoteRepo: true}); err != nil {
		return err
	}
	client := ctx.Login.Client()

	if !ctx.Args().Present() {
		fmt.Println("Release tag needed to edit")
		return nil
	}

	if !ctx.Bool("confirm") {
		fmt.Println("Are you sure? Please confirm with -y or --confirm.")
		return nil
	}

	for _, tag := range ctx.Args().Slice() {
		release, err := GetReleaseByTag(requestCtx, ctx.Owner, ctx.Repo, tag, client)
		if err != nil {
			return err
		}
		_, err = client.Releases.DeleteRelease(requestCtx, ctx.Owner, ctx.Repo, release.ID)
		if err != nil {
			return err
		}

		if ctx.Bool("delete-tag") {
			_, err = client.Repositories.DeleteTag(requestCtx, ctx.Owner, ctx.Repo, tag)
			return err
		}
	}

	return nil
}
