// Copyright 2021 The Gitea Authors. All rights reserved.
// SPDX-License-Identifier: MIT

package cmd

import (
	stdctx "context"

	"github.com/urfave/cli/v3"

	"gitea.dev/tea/modules/context"
	"gitea.dev/tea/modules/print"
)

// CmdWhoami represents the command to show current logged in user
var CmdWhoami = cli.Command{
	Name:        "whoami",
	Category:    catMisc,
	Description: `For debugging purposes, show the user that is currently logged in.`,
	Usage:       "Show current logged in user",
	ArgsUsage:   " ", // command does not accept arguments
	Action: func(requestCtx stdctx.Context, cmd *cli.Command) error {
		ctx, err := context.InitCommand(cmd)
		if err != nil {
			return err
		}
		client := ctx.Login.Client()
		user, _, _ := client.Users.GetMyUserInfo(requestCtx)
		print.UserDetails(user)
		return nil
	},
}
